@@ -5,7 +5,7 @@
 ## Look at bin/mce_grep for a wrapper script around the grep binary.
 ##
 ## This script supports egrep's options [ceHhiLlmnqRrsv]. The focus is
-## demonstrating Many-core Engine for Perl. Use this script against large
+## demonstrating Many-Core Engine for Perl. Use this script against large
 ## file(s).
 ##
 ## This script was created to show how output order can be preserved even
@@ -29,17 +29,14 @@
 use strict;
 use warnings;
 
-use Cwd qw(abs_path);
-use lib abs_path . "/../lib";
+use Cwd 'abs_path'; ## Insert lib-path at the head of @INC.
+use lib abs_path($0 =~ m{^(.*)[\\/]} && $1 || abs_path) . '/../lib';
 
 my ($prog_name, $prog_dir);
 
 BEGIN {
-   $prog_name = $0;
-   $prog_name =~ s{^.*[\\/]}{}g;
-
-   $prog_dir  = abs_path($0);
-   $prog_dir  =~ s{[\\/][^\\/]*$}{};
+   $prog_name = $0;             $prog_name =~ s{^.*[\\/]}{}g;
+   $prog_dir  = abs_path($0);   $prog_dir  =~ s{[\\/][^\\/]*$}{};
 
    $ENV{PATH} .= ($^O eq 'MSWin32' ? ';' : ':') . $prog_dir;
 }
@@ -60,12 +57,14 @@ use MCE;
 
 sub usage {
 
-   my $exit_status = $_[0] || 0;
+   my ($exit_status) = @_;
+   
+   $exit_status = 0 unless defined $exit_status;
 
    print <<"::_USAGE_BLOCK_END_::";
 
-Options for Many-core Engine:
-  --max-workers=NUM         override max workers (default 6)
+Options for Many-Core Engine:
+  --max-workers=NUM         override max workers (default auto)
                               e.g. auto, auto-2, 4
 
   --chunk-size=NUM[KM]      override chunk size (default 4M)
@@ -122,7 +121,7 @@ my ($c_flag, $H_flag, $h_flag, $i_flag, $n_flag, $q_flag, $r_flag, $v_flag);
 my (@r_patn, $arg, @files, @patterns, $re, $skip_args, $w_filename);
 my ($L_flag, $l_flag, $f_list);
 
-my $max_workers = 6; my $chunk_size = 4194304;  ## 4M
+my $max_workers = 'auto'; my $chunk_size = 4194304;  ## 4M
 my $max_count = 0; my $no_msg = 0;
 
 ## Option parsing step 1.
@@ -265,13 +264,13 @@ while ( @ARGV ) {
 {
    if (defined $max_count) {
       unless (looks_like_number($max_count) && $max_count >= 0) {
-         print STDERR "$prog_name: invalid max count\n";
+         print {*STDERR} "$prog_name: invalid max count\n";
          exit 2;
       }
    }
    if ($max_workers !~ /^auto/) {
       unless (looks_like_number($max_workers) && $max_workers > 0) {
-         print STDERR "$prog_name: invalid max workers\n";
+         print {*STDERR} "$prog_name: invalid max workers\n";
          exit 2;
       }
    }
@@ -288,7 +287,7 @@ while ( @ARGV ) {
       $chunk_size =    204_800 if $chunk_size <    204_800;  ## 200K
    }
    else {
-      print STDERR "$prog_name: invalid chunk size\n";
+      print {*STDERR} "$prog_name: invalid chunk size\n";
       exit 2;
    }
 }
@@ -338,7 +337,7 @@ sub aggregate_count {
 
 sub display_result {
 
-   my ($result, $chunk_id) = @_;
+   my ($chunk_id, $result) = @_;
 
    return if ($abort_job);
    $result{$chunk_id} = $result;
@@ -351,7 +350,7 @@ sub display_result {
          $found_match = 1;
 
          if ($q_flag) {
-            MCE->abort(); $abort_all = $abort_job = 1;
+            MCE->abort; $abort_all = $abort_job = 1;
             last;
          }
          for my $i (0 .. @{ $r->{matches} } - 1) {
@@ -364,23 +363,23 @@ sub display_result {
             }
 
             if ($max_count && $max_count == $total_found) {
-               MCE->abort(); $abort_job = 1;
+               MCE->abort; $abort_job = 1;
                last;
             }
          }
       }
 
       $total_lines += $r->{line_count} if ($n_flag);
-
-      delete $result{$order_id};
-      $order_id++;
+      delete $result{$order_id++};
    }
+
+   return;
 }
 
 sub report_match {
 
    if (!$abort_job) {
-      MCE->abort();
+      MCE->abort;
       $abort_all = 1 if $q_flag;
       $abort_job = $total_found = 1;
    }
@@ -534,7 +533,7 @@ sub user_func {
       'lines' => \@lines
    );
 
-   MCE->do('display_result', \%result, $chunk_id);
+   MCE->do('display_result', $chunk_id, \%result);
 
    return;
 }
@@ -564,16 +563,16 @@ sub display_matched {
 
 sub process_file {
 
-   $file = $_[0];
+   ($file) = @_;
 
    if ($file eq '-') {
-      open(STDIN, ($^O eq 'MSWin32') ? 'CON' : '/dev/tty') or die $!;
+      open(STDIN, '<', ($^O eq 'MSWin32') ? 'CON' : '/dev/tty') or die $!;
       process_stdin();
    }
    elsif (! -e $file) {
       $exit_status = 2;
 
-      print STDERR "$prog_name: $file: No such file or directory\n"
+      print {*STDERR} "$prog_name: $file: No such file or directory\n"
          unless $no_msg;
    }
    elsif (-d $file) {
@@ -614,13 +613,13 @@ if ($r_flag && @files > 0) {
 
    MCE->spawn;
 
-   unless ($^O eq 'MSWin32') {
-      open $list_fh, '-|', 'egrep', '-lsr', @r_patn, '^', @files;
-   }
-   else {
+   if ($^O eq 'MSWin32') {
       $list = `egrep -lsr @r_patn ^ @files`;
       open $list_fh, '<', \$list;
    }
+   else {
+      open $list_fh, '-|', 'egrep', '-lsr', @r_patn, '^', @files;
+   }
 
    while (<$list_fh>) {
       chomp;
@@ -646,7 +645,7 @@ else {
 ##
 ###############################################################################
 
-MCE->shutdown();
+MCE->shutdown;
 
 if (!$q_flag && $exit_status) {
    exit($exit_status);
