@@ -13,18 +13,19 @@
 use strict;
 use warnings;
 
-my ($prog_name, $prog_dir, $base_dir);
+## no critic (InputOutput::ProhibitBarewordFileHandles)
+## no critic (InputOutput::ProhibitTwoArgOpen)
+
+use Cwd 'abs_path'; ## Insert lib-path at the head of @INC.
+use lib abs_path($0 =~ m{^(.*)[\\/]} && $1 || abs_path) . '/../lib';
+
+my ($prog_name, $prog_dir);
 
 BEGIN {
-   use Cwd qw(abs_path);
-
    $prog_name = $0;             $prog_name =~ s{^.*[\\/]}{}g;
    $prog_dir  = abs_path($0);   $prog_dir  =~ s{[\\/][^\\/]*$}{};
-   $base_dir  = $prog_dir;      $base_dir  =~ s{[\\/][^\\/]*$}{};
 
    $ENV{PATH} = $prog_dir .($^O eq 'MSWin32' ? ';' : ':'). $ENV{PATH};
-
-   unshift @INC, "$base_dir/lib";
 }
 
 use Getopt::Long qw(
@@ -45,7 +46,7 @@ use MCE::Loop;
 
 sub usage {
 
-   print STDERR <<"::_USAGE_BLOCK_END_::";
+   print {*STDERR} <<"::_USAGE_BLOCK_END_::";
 
 NAME
    $prog_name -- process STDIN or FILE via an external command in parallel
@@ -137,7 +138,7 @@ my $parallel_io  = 0;
 
    if ($max_workers !~ /^auto/) {
       unless (looks_like_number($max_workers) && $max_workers > 0) {
-         print STDERR "$prog_name: $max_workers: invalid max workers\n";
+         print {*STDERR} "$prog_name: $max_workers: invalid max workers\n";
          exit 2;
       }
    }
@@ -151,7 +152,7 @@ my $parallel_io  = 0;
       }
 
       if (!looks_like_number($chunk_size) || $chunk_size < 1) {
-         print STDERR "$prog_name: $chunk_size: invalid chunk size\n";
+         print {*STDERR} "$prog_name: $chunk_size: invalid chunk size\n";
          exit 2;
       }
    }
@@ -166,14 +167,14 @@ usage() if $ARGV[0] =~ /^-/;
 ##
 ###############################################################################
 
-my $is_MSWin32 = $^O eq 'MSWin32';
+my $is_mswin32 = $^O eq 'MSWin32';
 my ($cmd_name, $cmd_path);
 
 $cmd_name = shift @ARGV;
 
 {
-   my $pth_sep = $is_MSWin32 ? ";"  : ":";
-   my $dir_sep = $is_MSWin32 ? "\\" : "/";
+   my $pth_sep = $is_mswin32 ? ";"  : ":";
+   my $dir_sep = $is_mswin32 ? "\\" : "/";
 
    for ( split ${pth_sep}, $ENV{'PATH'} ) {
       if (-e "$_${dir_sep}${cmd_name}" || -e "$_${dir_sep}${cmd_name}.exe") {
@@ -184,11 +185,11 @@ $cmd_name = shift @ARGV;
 }
 
 if (! defined $cmd_path) {
-   print STDERR "$prog_name: $cmd_name: command not found\n";
+   print {*STDERR} "$prog_name: $cmd_name: command not found\n";
    exit 2;
 }
-if (! $is_MSWin32 && ! -x $cmd_path) {
-   print STDERR "$prog_name: $cmd_name: command is not executable\n";
+if (! $is_mswin32 && ! -x $cmd_path) {
+   print {*STDERR} "$prog_name: $cmd_name: command is not executable\n";
    exit 2;
 }
 
@@ -201,11 +202,11 @@ if (defined $ARGV[-1] && $ARGV[-1] !~ /^-/) {
 
 if (ref $input eq '') {
    if (! -e $input) {
-      print STDERR "$prog_name: $input: No such file or directory\n";
+      print {*STDERR} "$prog_name: $input: No such file or directory\n";
       exit 2;
    }
    if (-d $input) {
-      print STDERR "$prog_name: $input: Is a directory\n";
+      print {*STDERR} "$prog_name: $input: Is a directory\n";
       exit 2;
    }
 }
@@ -238,28 +239,29 @@ sub output {
    }
 
    unlink $file;
+
+   return;
 }
 
 sub gather_iterator {
 
    my ($out_fh, $err_fh) = @_;
-   my (%tmp, $path); my $order_id = 1;
+   my %tmp; my $order_id = 1;
 
    return sub {
+      my ($chunk_id, $path, $status) = @_;
 
-      $tmp{$_[0]}  = $_[1]; 
-      $exit_status = $_[2] if ($_[2] > $exit_status);
+      $tmp{$chunk_id} = $path; 
+      $exit_status = $status if ($status > $exit_status);
 
-      MCE->abort() if ($abort_on_err && $_[2]);
+      MCE->abort if ($abort_on_err && $status);
 
       while (1) {
          last unless exists $tmp{$order_id};
 
-         $path = $tmp{$order_id};
+         $path = delete $tmp{$order_id++};
          output("$path.err", $err_fh);
          output("$path.out", $out_fh);
-
-         delete $tmp{$order_id++};
       }
    };
 }
@@ -282,10 +284,10 @@ MCE::Loop::init {
 mce_loop_f {
 
    my ($mce, $chunk_ref, $chunk_id) = @_;
-   my $path = MCE->tmp_dir() .'/'. $chunk_id;
+   my $path = MCE->tmp_dir .'/'. $chunk_id;
    local ($!, $?);
 
-   if ($is_MSWin32) {
+   if ($is_mswin32) {
       $path =~ s{/}{\\\\}g;
 
       open my $out_fh, "+>:raw", "$path.in";
@@ -332,7 +334,7 @@ mce_loop_f {
 ##
 ###############################################################################
 
-MCE::Loop::finish();
+MCE::Loop::finish;
 
 exit $exit_status;
 
