@@ -12,19 +12,18 @@
 
 package MCE::Core::Input::Generator;
 
-our $VERSION = '1.517'; $VERSION = eval $VERSION;
+use strict;
+use warnings;
+
+our $VERSION = '1.608';
 
 ## Items below are folded into MCE.
 
 package MCE;
 
-use strict;
-use warnings;
-
-## Warnings are disabled to minimize bits of noise when user or OS signals
-## the script to exit. e.g. MCE_script.pl < infile | head
-
-no warnings 'threads'; no warnings 'uninitialized';
+no warnings 'threads';
+no warnings 'recursion';
+no warnings 'uninitialized';
 
 ###############################################################################
 ## ----------------------------------------------------------------------------
@@ -34,13 +33,11 @@ no warnings 'threads'; no warnings 'uninitialized';
 
 sub _worker_sequence_generator {
 
-   my MCE $self = $_[0];
+   my ($self) = @_;
 
    @_ = ();
 
-   die "Private method called" unless (caller)[0]->isa( ref($self) );
-
-   _croak("MCE::_worker_sequence_generator: 'user_func' is not specified")
+   _croak('MCE::_worker_sequence_generator: (user_func) is not specified')
       unless (defined $self->{user_func});
 
    my $_bounds_only = $self->{bounds_only} || 0;
@@ -68,6 +65,8 @@ sub _worker_sequence_generator {
 
    ## -------------------------------------------------------------------------
 
+   local $_;
+
    $self->{_last_jmp} = sub { goto _WORKER_SEQ_GEN__LAST; };
 
    if ($_begin == $_end) {                        ## Identical, yes.
@@ -75,7 +74,7 @@ sub _worker_sequence_generator {
       if ($_wid == 1) {
          $self->{_next_jmp} = sub { goto _WORKER_SEQ_GEN__LAST; };
 
-         local $_ = (defined $_fmt) ? sprintf("%$_fmt", $_next) : $_next;
+         $_ = (defined $_fmt) ? sprintf("%$_fmt", $_next) : $_next;
 
          if ($_chunk_size > 1) {
             $_ = ($_bounds_only) ? [ $_, $_ ] : [ $_ ];
@@ -94,7 +93,7 @@ sub _worker_sequence_generator {
          return if ( $_flag && $_next > $_end);
          return if (!$_flag && $_next < $_end);
 
-         local $_ = (defined $_fmt) ? sprintf("%$_fmt", $_next) : $_next;
+         $_ = (defined $_fmt) ? sprintf("%$_fmt", $_next) : $_next;
          $_wuf->($self, $_, $_chunk_id);
 
          _WORKER_SEQ_GEN__NEXT_A:
@@ -108,7 +107,7 @@ sub _worker_sequence_generator {
       $self->{_next_jmp} = sub { goto _WORKER_SEQ_GEN__NEXT_B; };
 
       while (1) {
-         my $_n_begin = $_next; my @_n = ();
+         my @_n = (); my $_n_begin = $_next;
 
          ## -------------------------------------------------------------------
 
@@ -125,10 +124,10 @@ sub _worker_sequence_generator {
                   );
                   $_start = 1 if ($_start < 1);
 
-                  for ($_start .. $_chunk_size) {
+                  for my $_i ($_start .. $_chunk_size) {
                      last if ($_next > $_end);
                      $_tmp_e = $_next;
-                     $_next  = $_step * $_ + $_n_begin;
+                     $_next  = $_step * $_i + $_n_begin;
                   }
                }
             }
@@ -142,10 +141,10 @@ sub _worker_sequence_generator {
                   );
                   $_start = 1 if ($_start < 1);
 
-                  for ($_start .. $_chunk_size) {
+                  for my $_i ($_start .. $_chunk_size) {
                      last if ($_next < $_end);
                      $_tmp_e = $_next;
-                     $_next  = $_step * $_ + $_n_begin;
+                     $_next  = $_step * $_i + $_n_begin;
                   }
                }
             }
@@ -171,33 +170,33 @@ sub _worker_sequence_generator {
                   }
                }
                else {
-                  for (1 .. $_chunk_size) {
+                  for my $_i (1 .. $_chunk_size) {
                      last if ($_next > $_end);
 
                      push @_n, (defined $_fmt)
                         ? sprintf("%$_fmt", $_next) : $_next;
 
-                     $_next = $_step * $_ + $_n_begin;
+                     $_next = $_step * $_i + $_n_begin;
                   }
                }
             }
             else {
-               for (1 .. $_chunk_size) {
+               for my $_i (1 .. $_chunk_size) {
                   last if ($_next < $_end);
 
                   push @_n, (defined $_fmt)
                      ? sprintf("%$_fmt", $_next) : $_next;
 
-                  $_next = $_step * $_ + $_n_begin;
+                  $_next = $_step * $_i + $_n_begin;
                }
             }
 
-            return unless (@_n > 0);
+            return unless (scalar @_n);
          }
 
          ## -------------------------------------------------------------------
 
-         local $_ = \@_n;
+         $_ = \@_n;
          $_wuf->($self, \@_n, $_chunk_id);
 
          _WORKER_SEQ_GEN__NEXT_B:
