@@ -11,19 +11,18 @@
 
 package MCE::Core::Input::Iterator;
 
-our $VERSION = '1.517'; $VERSION = eval $VERSION;
+use strict;
+use warnings;
+
+our $VERSION = '1.608';
 
 ## Items below are folded into MCE.
 
 package MCE;
 
-use strict;
-use warnings;
-
-## Warnings are disabled to minimize bits of noise when user or OS signals
-## the script to exit. e.g. MCE_script.pl < infile | head
-
-no warnings 'threads'; no warnings 'uninitialized';
+no warnings 'threads';
+no warnings 'recursion';
+no warnings 'uninitialized';
 
 ###############################################################################
 ## ----------------------------------------------------------------------------
@@ -33,13 +32,11 @@ no warnings 'threads'; no warnings 'uninitialized';
 
 sub _worker_user_iterator {
 
-   my MCE $self = $_[0];
+   my ($self) = @_;
 
    @_ = ();
 
-   die "Private method called" unless (caller)[0]->isa( ref($self) );
-
-   _croak("MCE::_worker_user_iterator: 'user_func' is not specified")
+   _croak('MCE::_worker_user_iterator: (user_func) is not specified')
       unless (defined $self->{user_func});
 
    my $_chn         = $self->{_chn};
@@ -51,28 +48,28 @@ sub _worker_user_iterator {
    my $_I_FLG       = (!$/ || $/ ne $LF);
    my $_wuf         = $self->{_wuf};
 
-   my ($_chunk_id, $_len, $_chunk_ref, $_is_ref);
+   my ($_chunk_id, $_len, $_is_ref);
 
    ## -------------------------------------------------------------------------
 
    $self->{_next_jmp} = sub { goto _WORKER_USER_ITERATOR__NEXT; };
    $self->{_last_jmp} = sub { goto _WORKER_USER_ITERATOR__LAST; };
 
+   local $_;
+
    _WORKER_USER_ITERATOR__NEXT:
 
    while (1) {
+      undef $_ if (length > MAX_CHUNK_SIZE);
 
-      ## Don't declare $_buffer with other vars above, instead it's done here.
-      ## Doing so will fail with Perl 5.8.0 under Solaris 5.10 on large files.
-
-      my $_buffer;
+      $_ = '';
 
       ## Obtain the next chunk of data.
       {
          local $\ = undef if (defined $\); local $/ = $LF if ($_I_FLG);
 
          flock $_DAT_LOCK, LOCK_EX if ($_lock_chn);
-         print $_DAT_W_SOCK OUTPUT_U_ITR . $LF . $_chn . $LF;
+         print {$_DAT_W_SOCK} OUTPUT_U_ITR . $LF . $_chn . $LF;
          chomp($_len = <$_DAU_W_SOCK>);
 
          if ($_len < 0) {
@@ -83,20 +80,20 @@ sub _worker_user_iterator {
          $_is_ref = chop $_len;
 
          chomp($_chunk_id = <$_DAU_W_SOCK>);
-         read $_DAU_W_SOCK, $_buffer, $_len;
+         read $_DAU_W_SOCK, $_, $_len;
 
          flock $_DAT_LOCK, LOCK_UN if ($_lock_chn);
       }
 
       ## Call user function.
       if ($_is_ref) {
-         $_chunk_ref = $self->{thaw}($_buffer); undef $_buffer;
-      } else {
-         $_chunk_ref = [ $_buffer ];
+         my $_chunk_ref = $self->{thaw}($_); undef $_;
+         $_ = ($_chunk_size == 1) ? $_chunk_ref->[0] : $_chunk_ref;
+         $_wuf->($self, $_chunk_ref, $_chunk_id);
+      }
+      else {
+         $_wuf->($self, [ $_ ], $_chunk_id);
       }
-
-      local $_ = ($_chunk_size == 1) ? $_chunk_ref->[0] : $_chunk_ref;
-      $_wuf->($self, $_chunk_ref, $_chunk_id);
    }
 
    _WORKER_USER_ITERATOR__LAST:
