@@ -11,19 +11,18 @@
 
 package MCE::Core::Input::Request;
 
-our $VERSION = '1.517'; $VERSION = eval $VERSION;
+use strict;
+use warnings;
+
+our $VERSION = '1.608';
 
 ## Items below are folded into MCE.
 
 package MCE;
 
-use strict;
-use warnings;
-
-## Warnings are disabled to minimize bits of noise when user or OS signals
-## the script to exit. e.g. MCE_script.pl < infile | head
-
-no warnings 'threads'; no warnings 'uninitialized';
+no warnings 'threads';
+no warnings 'recursion';
+no warnings 'uninitialized';
 
 ###############################################################################
 ## ----------------------------------------------------------------------------
@@ -33,13 +32,11 @@ no warnings 'threads'; no warnings 'uninitialized';
 
 sub _worker_request_chunk {
 
-   my MCE $self = $_[0]; my $_proc_type = $_[1];
+   my ($self, $_proc_type) = @_;
 
    @_ = ();
 
-   die "Private method called" unless (caller)[0]->isa( ref($self) );
-
-   _croak("MCE::_worker_request_chunk: 'user_func' is not specified")
+   _croak('MCE::_worker_request_chunk: (user_func) is not specified')
       unless (defined $self->{user_func});
 
    my $_chn         = $self->{_chn};
@@ -55,14 +52,22 @@ sub _worker_request_chunk {
    my $_I_FLG       = (!$/ || $/ ne $LF);
    my $_wuf         = $self->{_wuf};
 
-   my ($_chunk_id, $_len, $_chunk_ref);
-   my ($_output_tag, @_records);
+   my ($_chunk_id, $_len, $_output_tag);
+   my ($_chop_len, $_chop_str, $_p);
 
    if ($_proc_type == REQUEST_ARRAY) {
       $_output_tag = OUTPUT_A_ARY;
-   } else {
+      $_chop_len   = 0;
+   }
+   else {
       $_output_tag = OUTPUT_S_GLB;
-      @_records    = ();
+      if (length $_RS > 1 && substr($_RS, 0, 1) eq "\n") {
+         $_chop_str = substr($_RS, 1);
+         $_chop_len = length $_chop_str;
+      } else {
+         $_chop_str = '';
+         $_chop_len = 0;
+      }
    }
 
    ## -------------------------------------------------------------------------
@@ -70,21 +75,21 @@ sub _worker_request_chunk {
    $self->{_next_jmp} = sub { goto _WORKER_REQUEST_CHUNK__NEXT; };
    $self->{_last_jmp} = sub { goto _WORKER_REQUEST_CHUNK__LAST; };
 
+   local $_;
+
    _WORKER_REQUEST_CHUNK__NEXT:
 
    while (1) {
+      undef $_ if (length > MAX_CHUNK_SIZE);
 
-      ## Don't declare $_buffer with other vars above, instead it's done here.
-      ## Doing so will fail with Perl 5.8.0 under Solaris 5.10 on large files.
-
-      my $_buffer;
+      $_ = '';
 
       ## Obtain the next chunk of data.
       {
          local $\ = undef if (defined $\); local $/ = $LF if ($_I_FLG);
 
          flock $_DAT_LOCK, LOCK_EX if ($_lock_chn);
-         print $_DAT_W_SOCK $_output_tag . $LF . $_chn . $LF;
+         print {$_DAT_W_SOCK} $_output_tag . $LF . $_chn . $LF;
          chomp($_len = <$_DAU_W_SOCK>);
 
          unless ($_len) {
@@ -93,7 +98,14 @@ sub _worker_request_chunk {
          }
 
          chomp($_chunk_id = <$_DAU_W_SOCK>);
-         read $_DAU_W_SOCK, $_buffer, $_len;
+
+         if ($_chunk_id > 1 && $_chop_len) {
+            $_p = $_chop_len; $_ = $_chop_str;
+         } else {
+            $_p = 0;
+         }
+
+         read $_DAU_W_SOCK, $_, $_len, $_p;
 
          flock $_DAT_LOCK, LOCK_UN if ($_lock_chn);
       }
@@ -101,32 +113,45 @@ sub _worker_request_chunk {
       ## Call user function.
       if ($_proc_type == REQUEST_ARRAY) {
          if ($_single_dim && $_chunk_size == 1) {
-            local $_ = $_buffer;
-            $_wuf->($self, [ $_buffer ], $_chunk_id);
+            $_wuf->($self, [ $_ ], $_chunk_id);
          }
          else {
-            $_chunk_ref = $self->{thaw}($_buffer); undef $_buffer;
-            local $_ = ($_chunk_size == 1) ? $_chunk_ref->[0] : $_chunk_ref;
+            my $_chunk_ref = $self->{thaw}($_); undef $_;
+            $_ = ($_chunk_size == 1) ? $_chunk_ref->[0] : $_chunk_ref;
             $_wuf->($self, $_chunk_ref, $_chunk_id);
          }
       }
       else {
          if ($_use_slurpio) {
-            local $_ = \$_buffer;
-            $_wuf->($self, \$_buffer, $_chunk_id);
+            if ($_chop_len && substr($_, -$_chop_len) eq $_chop_str) {
+               substr($_, -$_chop_len, $_chop_len, '');
+            }
+            local $_ = \$_;
+            $_wuf->($self, $_, $_chunk_id);
          }
          else {
             if ($_chunk_size == 1) {
-               local $_ = $_buffer;
-               $_wuf->($self, [ $_buffer ], $_chunk_id);
+               if ($_chop_len && substr($_, -$_chop_len) eq $_chop_str) {
+                  substr($_, -$_chop_len, $_chop_len, '');
+               }
+               $_wuf->($self, [ $_ ], $_chunk_id);
             }
             else {
+               my @_recs;
                {
                   local $/ = $_RS if ($_RS_FLG);
-                  _sync_buffer_to_array(\$_buffer, \@_records);
+                  _sync_buffer_to_array(\$_, \@_recs, $_chop_str);
+                  undef $_;
                }
-               local $_ = \@_records;
-               $_wuf->($self, \@_records, $_chunk_id);
+               if ($_chop_len) {
+                  for my $i (0 .. @_recs - 1) {
+                     if (substr($_recs[$i], -$_chop_len) eq $_chop_str) {
+                        substr($_recs[$i], -$_chop_len, $_chop_len, '');
+                     }
+                  }
+               }
+               local $_ = \@_recs;
+               $_wuf->($self, \@_recs, $_chunk_id);
             }
          }
       }
