@@ -11,19 +11,18 @@
 
 package MCE::Core::Input::Sequence;
 
-our $VERSION = '1.517'; $VERSION = eval $VERSION;
+use strict;
+use warnings;
+
+our $VERSION = '1.608';
 
 ## Items below are folded into MCE.
 
 package MCE;
 
-use strict;
-use warnings;
-
-## Warnings are disabled to minimize bits of noise when user or OS signals
-## the script to exit. e.g. MCE_script.pl < infile | head
-
-no warnings 'threads'; no warnings 'uninitialized';
+no warnings 'threads';
+no warnings 'recursion';
+no warnings 'uninitialized';
 
 my $_que_read_size = $MCE::_que_read_size;
 my $_que_template  = $MCE::_que_template;
@@ -36,13 +35,11 @@ my $_que_template  = $MCE::_que_template;
 
 sub _worker_sequence_queue {
 
-   my MCE $self = $_[0];
+   my ($self) = @_;
 
    @_ = ();
 
-   die "Private method called" unless (caller)[0]->isa( ref($self) );
-
-   _croak("MCE::_worker_sequence_queue: 'user_func' is not specified")
+   _croak('MCE::_worker_sequence_queue: (user_func) is not specified')
       unless (defined $self->{user_func});
 
    my $_QUE_R_SOCK  = $self->{_que_r_sock};
@@ -74,6 +71,10 @@ sub _worker_sequence_queue {
    $self->{_next_jmp} = sub { goto _WORKER_SEQUENCE__NEXT; };
    $self->{_last_jmp} = sub { goto _WORKER_SEQUENCE__LAST; };
 
+   local $_;
+
+   _WORKER_SEQUENCE__NEXT:
+
    while (1) {
 
       ## Obtain the next chunk_id and sequence number.
@@ -92,10 +93,9 @@ sub _worker_sequence_queue {
 
       ## Call user function.
       if ($_chunk_size == 1) {
-         $_seq_n = $_offset * $_step + $_begin;
-         $_seq_n = sprintf("%$_fmt", $_seq_n) if (defined $_fmt);
-         local $_ = $_seq_n;
-         $_wuf->($self, $_seq_n, $_chunk_id);
+         $_ = $_offset * $_step + $_begin;
+         $_ = sprintf("%$_fmt", $_) if (defined $_fmt);
+         $_wuf->($self, $_, $_chunk_id);
       }
       else {
          my $_n_begin = ($_offset * $_chunk_size) * $_step + $_begin;
@@ -116,10 +116,10 @@ sub _worker_sequence_queue {
                   );
                   $_start = 1 if ($_start < 1);
 
-                  for ($_start .. $_chunk_size) {
+                  for my $_i ($_start .. $_chunk_size) {
                      last if ($_seq_n > $_end);
                      $_tmp_e = $_seq_n;
-                     $_seq_n = $_step * $_ + $_n_begin;
+                     $_seq_n = $_step * $_i + $_n_begin;
                   }
                }
             }
@@ -133,10 +133,10 @@ sub _worker_sequence_queue {
                   );
                   $_start = 1 if ($_start < 1);
 
-                  for ($_start .. $_chunk_size) {
+                  for my $_i ($_start .. $_chunk_size) {
                      last if ($_seq_n < $_end);
                      $_tmp_e = $_seq_n;
-                     $_seq_n = $_step * $_ + $_n_begin;
+                     $_seq_n = $_step * $_i + $_n_begin;
                   }
                }
             }
@@ -153,7 +153,7 @@ sub _worker_sequence_queue {
          else {
             if ($_begin < $_end) {
                if (!defined $_fmt && $_step == 1) {
-                  local $_ = ($_seq_n + $_chunk_size <= $_end)
+                  $_ = ($_seq_n + $_chunk_size <= $_end)
                      ? [ $_seq_n .. $_seq_n + $_chunk_size - 1 ]
                      : [ $_seq_n .. $_end ];
 
@@ -161,35 +161,33 @@ sub _worker_sequence_queue {
                   next;
                }
                else {
-                  for (1 .. $_chunk_size) {
+                  for my $_i (1 .. $_chunk_size) {
                      last if ($_seq_n > $_end);
 
                      push @_n, (defined $_fmt)
                         ? sprintf("%$_fmt", $_seq_n) : $_seq_n;
 
-                     $_seq_n = $_step * $_ + $_n_begin;
+                     $_seq_n = $_step * $_i + $_n_begin;
                   }
                }
             }
             else {
-               for (1 .. $_chunk_size) {
+               for my $_i (1 .. $_chunk_size) {
                   last if ($_seq_n < $_end);
 
                   push @_n, (defined $_fmt)
                      ? sprintf("%$_fmt", $_seq_n) : $_seq_n;
 
-                  $_seq_n = $_step * $_ + $_n_begin;
+                  $_seq_n = $_step * $_i + $_n_begin;
                }
             }
          }
 
          ## -------------------------------------------------------------------
 
-         local $_ = \@_n;
+         $_ = \@_n;
          $_wuf->($self, \@_n, $_chunk_id);
       }
-
-      _WORKER_SEQUENCE__NEXT:
    }
 
    _WORKER_SEQUENCE__LAST:
