@@ -11,28 +11,21 @@
 
 package MCE::Core::Worker;
 
-our $VERSION = '1.517'; $VERSION = eval $VERSION;
+use strict;
+use warnings;
+
+our $VERSION = '1.608';
 
 ## Items below are folded into MCE.
 
 package MCE;
 
-use strict;
-use warnings;
+no warnings 'threads';
+no warnings 'recursion';
+no warnings 'uninitialized';
 
 use bytes;
 
-## Warnings are disabled to minimize bits of noise when user or OS signals
-## the script to exit. e.g. MCE_script.pl < infile | head
-
-no warnings 'threads'; no warnings 'uninitialized';
-
-my $_die_msg;
-
-END {
-   MCE->exit(255, $_die_msg) if (defined $_die_msg);
-}
-
 ###############################################################################
 ## ----------------------------------------------------------------------------
 ## Internal do, gather and send related functions for serializing data to
@@ -41,8 +34,10 @@ END {
 ###############################################################################
 
 {
-   my ($_DAT_LOCK, $_DAT_W_SOCK, $_DAU_W_SOCK, $_tag, $_value, $_want_id);
-   my ($_chn, $_data_ref, $_dest, $_len, $_lock_chn, $_task_id, $_user_func);
+   my (
+      $_dest, $_len, $_tag, $_task_id, $_user_func, $_value, $_want_id,
+      $_DAT_LOCK, $_DAT_W_SOCK, $_DAU_W_SOCK, $_lock_chn, $_chn
+   );
 
    ## Create array structure containing various send functions.
    my @_dest_function = ();
@@ -50,15 +45,15 @@ END {
    $_dest_function[SENDTO_FILEV2] = sub {         ## Content >> File
 
       return unless (defined $_value);
-
-      my $_buffer = $_value . $LF . length(${ $_[0] }) . $LF . ${ $_[0] };
-
       local $\ = undef if (defined $\);
 
-      flock $_DAT_LOCK, LOCK_EX if ($_lock_chn);
-      print $_DAT_W_SOCK OUTPUT_F_SND . $LF . $_chn . $LF;
-      print $_DAU_W_SOCK $_buffer;
-      flock $_DAT_LOCK, LOCK_UN if ($_lock_chn);
+      if (length ${ $_[0] }) {
+         flock $_DAT_LOCK, LOCK_EX if ($_lock_chn);
+         print {$_DAT_W_SOCK} OUTPUT_F_SND . $LF . $_chn . $LF;
+         print {$_DAU_W_SOCK} $_value . $LF . length(${ $_[0] }) . $LF;
+         print {$_DAU_W_SOCK} ${ $_[0] };
+         flock $_DAT_LOCK, LOCK_UN if ($_lock_chn);
+      }
 
       return;
    };
@@ -66,43 +61,45 @@ END {
    $_dest_function[SENDTO_FD] = sub {             ## Content >> File descriptor
 
       return unless (defined $_value);
-
-      my $_buffer = $_value . $LF . length(${ $_[0] }) . $LF . ${ $_[0] };
-
       local $\ = undef if (defined $\);
 
-      flock $_DAT_LOCK, LOCK_EX if ($_lock_chn);
-      print $_DAT_W_SOCK OUTPUT_D_SND . $LF . $_chn . $LF;
-      print $_DAU_W_SOCK $_buffer;
-      flock $_DAT_LOCK, LOCK_UN if ($_lock_chn);
+      if (length ${ $_[0] }) {
+         flock $_DAT_LOCK, LOCK_EX if ($_lock_chn);
+         print {$_DAT_W_SOCK} OUTPUT_D_SND . $LF . $_chn . $LF;
+         print {$_DAU_W_SOCK} $_value . $LF . length(${ $_[0] }) . $LF;
+         print {$_DAU_W_SOCK} ${ $_[0] };
+         flock $_DAT_LOCK, LOCK_UN if ($_lock_chn);
+      }
 
       return;
    };
 
    $_dest_function[SENDTO_STDOUT] = sub {         ## Content >> STDOUT
 
-      my $_buffer = length(${ $_[0] }) . $LF . ${ $_[0] };
-
       local $\ = undef if (defined $\);
 
-      flock $_DAT_LOCK, LOCK_EX if ($_lock_chn);
-      print $_DAT_W_SOCK OUTPUT_O_SND . $LF . $_chn . $LF;
-      print $_DAU_W_SOCK $_buffer;
-      flock $_DAT_LOCK, LOCK_UN if ($_lock_chn);
+      if (length ${ $_[0] }) {
+         flock $_DAT_LOCK, LOCK_EX if ($_lock_chn);
+         print {$_DAT_W_SOCK} OUTPUT_O_SND . $LF . $_chn . $LF;
+         print {$_DAU_W_SOCK} length(${ $_[0] }) . $LF;
+         print {$_DAU_W_SOCK} ${ $_[0] };
+         flock $_DAT_LOCK, LOCK_UN if ($_lock_chn);
+      }
 
       return;
    };
 
    $_dest_function[SENDTO_STDERR] = sub {         ## Content >> STDERR
 
-      my $_buffer = length(${ $_[0] }) . $LF . ${ $_[0] };
-
       local $\ = undef if (defined $\);
 
-      flock $_DAT_LOCK, LOCK_EX if ($_lock_chn);
-      print $_DAT_W_SOCK OUTPUT_E_SND . $LF . $_chn . $LF;
-      print $_DAU_W_SOCK $_buffer;
-      flock $_DAT_LOCK, LOCK_UN if ($_lock_chn);
+      if (length ${ $_[0] }) {
+         flock $_DAT_LOCK, LOCK_EX if ($_lock_chn);
+         print {$_DAT_W_SOCK} OUTPUT_E_SND . $LF . $_chn . $LF;
+         print {$_DAU_W_SOCK} length(${ $_[0] }) . $LF;
+         print {$_DAU_W_SOCK} ${ $_[0] };
+         flock $_DAT_LOCK, LOCK_UN if ($_lock_chn);
+      }
 
       return;
    };
@@ -111,9 +108,7 @@ END {
 
    sub _do_callback {
 
-      my $_buffer; my MCE $self = $_[0]; $_value = $_[1]; $_data_ref = $_[2];
-
-      @_ = ();
+      my ($self, $_buf, $_aref);  ($self, $_value, $_aref) = @_;
 
       unless (defined wantarray) {
          $_want_id = WANTS_UNDEF;
@@ -125,33 +120,37 @@ END {
 
       ## Crossover: Send arguments
 
-      if (@$_data_ref > 0) {                      ## Multiple Args >> Callback
-
-         if (@$_data_ref > 1 || ref $_data_ref->[0]) {
+      if (scalar @{ $_aref } > 0) {               ## Multiple Args >> Callback
+         if (scalar @{ $_aref } > 1 || ref $_aref->[0]) {
             $_tag = OUTPUT_A_CBK;
-            $_buffer = $self->{freeze}($_data_ref);
-            $_buffer = $_want_id . $LF . $_value . $LF .
-               length($_buffer) . $LF . $_buffer;
+            $_buf = $self->{freeze}($_aref);
+            $_len = length $_buf; local $\ = undef if (defined $\);
+
+            flock $_DAT_LOCK, LOCK_EX if ($_lock_chn);
+            print {$_DAT_W_SOCK} $_tag . $LF . $_chn . $LF;
+            print {$_DAU_W_SOCK} $_want_id . $LF . $_value . $LF . $_len . $LF;
+            print {$_DAU_W_SOCK} $_buf;
+
          }
          else {                                   ## Scalar >> Callback
             $_tag = OUTPUT_S_CBK;
-            $_buffer = $_want_id . $LF . $_value . $LF .
-               length($_data_ref->[0]) . $LF . $_data_ref->[0];
-         }
+            $_len = length $_aref->[0]; local $\ = undef if (defined $\);
 
-         $_data_ref = '';
+            flock $_DAT_LOCK, LOCK_EX if ($_lock_chn);
+            print {$_DAT_W_SOCK} $_tag . $LF . $_chn . $LF;
+            print {$_DAU_W_SOCK} $_want_id . $LF . $_value . $LF . $_len . $LF;
+            print {$_DAU_W_SOCK} $_aref->[0];
+         }
       }
       else {                                      ## No Args >> Callback
          $_tag = OUTPUT_N_CBK;
-         $_buffer = $_want_id . $LF . $_value . $LF;
+         local $\ = undef if (defined $\);
+
+         flock $_DAT_LOCK, LOCK_EX if ($_lock_chn);
+         print {$_DAT_W_SOCK} $_tag . $LF . $_chn . $LF;
+         print {$_DAU_W_SOCK} $_want_id . $LF . $_value . $LF;
       }
 
-      local $\ = undef if (defined $\);
-
-      flock $_DAT_LOCK, LOCK_EX if ($_lock_chn);
-      print $_DAT_W_SOCK $_tag . $LF . $_chn . $LF;
-      print $_DAU_W_SOCK $_buffer;
-
       ## Crossover: Receive return value
 
       if ($_want_id == WANTS_UNDEF) {
@@ -160,25 +159,24 @@ END {
       }
       elsif ($_want_id == WANTS_ARRAY) {
          local $/ = $LF if (!$/ || $/ ne $LF);
-
          chomp($_len = <$_DAU_W_SOCK>);
-         read $_DAU_W_SOCK, $_buffer, $_len || 0;
+
+         read($_DAU_W_SOCK, $_buf, $_len || 0);
          flock $_DAT_LOCK, LOCK_UN if ($_lock_chn);
 
-         return @{ $self->{thaw}($_buffer) };
+         return @{ $self->{thaw}($_buf) };
       }
       else {
          local $/ = $LF if (!$/ || $/ ne $LF);
-
          chomp($_want_id = <$_DAU_W_SOCK>);
          chomp($_len     = <$_DAU_W_SOCK>);
 
          if ($_len >= 0) {
-            read $_DAU_W_SOCK, $_buffer, $_len || 0;
+            read($_DAU_W_SOCK, $_buf, $_len || 0);
             flock $_DAT_LOCK, LOCK_UN if ($_lock_chn);
 
-            return $_buffer if ($_want_id == WANTS_SCALAR);
-            return $self->{thaw}($_buffer);
+            return $_buf if ($_want_id == WANTS_SCALAR);
+            return $self->{thaw}($_buf);
          }
          else {
             flock $_DAT_LOCK, LOCK_UN if ($_lock_chn);
@@ -191,38 +189,45 @@ END {
 
    sub _do_gather {
 
-      my MCE $self  = $_[0];
-      my $_data_ref = $_[1];
-      my $_buffer;
+      my $_buf; my ($self, $_aref) = @_;
 
-      return unless (scalar @{ $_data_ref });
+      return unless (scalar @{ $_aref });
 
-      if (@{ $_data_ref } > 1) {
+      if (scalar @{ $_aref } > 1) {
          $_tag = OUTPUT_A_GTR;
-         $_buffer = $self->{freeze}($_data_ref);
-         $_buffer = $_task_id . $LF . length($_buffer) . $LF . $_buffer;
+         $_buf = $self->{freeze}($_aref);
+         $_len = length $_buf;
       }
-      elsif (ref $_data_ref->[0]) {
+      elsif (ref $_aref->[0]) {
          $_tag = OUTPUT_R_GTR;
-         $_buffer = $self->{freeze}($_data_ref->[0]);
-         $_buffer = $_task_id . $LF . length($_buffer) . $LF . $_buffer;
+         $_buf = $self->{freeze}($_aref->[0]);
+         $_len = length $_buf;
       }
-      elsif (scalar @{ $_data_ref }) {
+      else {
          $_tag = OUTPUT_S_GTR;
-         if (defined $_data_ref->[0]) {
-            $_buffer = $_task_id . $LF . length($_data_ref->[0]) . $LF .
-               $_data_ref->[0];
+         if (defined $_aref->[0]) {
+            $_len = length $_aref->[0]; local $\ = undef if (defined $\);
+
+            flock $_DAT_LOCK, LOCK_EX if ($_lock_chn);
+            print {$_DAT_W_SOCK} $_tag . $LF . $_chn . $LF;
+            print {$_DAU_W_SOCK} $_task_id . $LF . $_len . $LF;
+            print {$_DAU_W_SOCK} $_aref->[0];
+            flock $_DAT_LOCK, LOCK_UN if ($_lock_chn);
+
+            return;
          }
          else {
-            $_buffer = $_task_id . $LF . -1 . $LF;
+            $_buf = '';
+            $_len = -1;
          }
       }
 
       local $\ = undef if (defined $\);
 
       flock $_DAT_LOCK, LOCK_EX if ($_lock_chn);
-      print $_DAT_W_SOCK $_tag . $LF . $_chn . $LF;
-      print $_DAU_W_SOCK $_buffer;
+      print {$_DAT_W_SOCK} $_tag . $LF . $_chn . $LF;
+      print {$_DAU_W_SOCK} $_task_id . $LF . $_len . $LF;
+      print {$_DAU_W_SOCK} $_buf if (length $_buf);
       flock $_DAT_LOCK, LOCK_UN if ($_lock_chn);
 
       return;
@@ -232,69 +237,89 @@ END {
 
    sub _do_send {
 
-      my MCE $self = shift; $_dest = shift; $_value = shift;
-      my $_buffer;
+      my $_data_ref; my $self = shift;
 
-      if (@_ > 1) {
-         $_buffer = join('', @_);
-         return $_dest_function[$_dest](\$_buffer);
+      $_dest = shift; $_value = shift;
+
+      if (scalar @_ > 1) {
+         $_data_ref = \join('', @_);
       }
       elsif (my $_ref = ref $_[0]) {
          if ($_ref eq 'SCALAR') {
-            return $_dest_function[$_dest]($_[0]);
-         } elsif ($_ref eq 'ARRAY') {
-            $_buffer = join('', @{ $_[0] });
-         } elsif ($_ref eq 'HASH') {
-            $_buffer = join('', %{ $_[0] });
-         } else {
-            $_buffer = join('', @_);
+            $_data_ref = $_[0];
+         }
+         elsif ($_ref eq 'ARRAY') {
+            $_data_ref = \join('', @{ $_[0] });
+         }
+         elsif ($_ref eq 'HASH') {
+            $_data_ref = \join('', %{ $_[0] });
+         }
+         else {
+            $_data_ref = \join('', @_);
          }
-         return $_dest_function[$_dest](\$_buffer);
       }
       else {
-         return $_dest_function[$_dest](\$_[0]);
+         $_data_ref = \$_[0];
       }
+
+      $_dest_function[$_dest]($_data_ref);
+
+      return;
    }
 
    sub _do_send_glob {
 
-      my MCE $self  = $_[0]; my $_glob = $_[1]; my $_fd = $_[2];
-      my $_data_ref = $_[3];
+      my ($self, $_glob, $_fd, $_data_ref) = @_;
 
       if ($self->{_wid} > 0) {
          if ($_fd == 1) {
-            $self->_do_send(SENDTO_STDOUT, undef, $_data_ref);
-         } elsif ($_fd == 2) {
-            $self->_do_send(SENDTO_STDERR, undef, $_data_ref);
-         } else {
-            $self->_do_send(SENDTO_FD, $_fd, $_data_ref);
+            _do_send($self, SENDTO_STDOUT, undef, $_data_ref);
+         }
+         elsif ($_fd == 2) {
+            _do_send($self, SENDTO_STDERR, undef, $_data_ref);
+         }
+         else {
+            _do_send($self, SENDTO_FD, $_fd, $_data_ref);
          }
       }
       else {
          my $_fh = qualify_to_ref($_glob, caller);
          local $\ = undef if (defined $\);
-         print $_fh $$_data_ref;
+         print {$_fh} ${ $_data_ref };
       }
 
       return;
    }
 
+   ## -------------------------------------------------------------------------
+
    sub _do_send_init {
 
-      my MCE $self = $_[0];
-
-      @_ = ();
-
-      die "Private method called" unless (caller)[0]->isa( ref($self) );
+      my ($self) = @_;
 
       $_chn        = $self->{_chn};
       $_DAT_LOCK   = $self->{_dat_lock};
       $_DAT_W_SOCK = $self->{_dat_w_sock}->[0];
       $_DAU_W_SOCK = $self->{_dat_w_sock}->[$_chn];
       $_lock_chn   = $self->{_lock_chn};
-
       $_task_id    = $self->{_task_id};
 
+      local ($!, $@);
+
+      eval { select STDERR; $| = 1 };
+      eval { select STDOUT; $| = 1 };
+
+      return;
+   }
+
+   sub _do_send_clear {
+
+      my ($self) = @_;
+
+      $_dest = $_len = $_task_id = $_user_func = $_value = $_want_id = undef;
+      $_DAT_LOCK = $_DAT_W_SOCK = $_DAU_W_SOCK = $_lock_chn = $_chn = undef;
+      $_tag = undef;
+
       return;
    }
 
@@ -302,12 +327,9 @@ END {
 
    sub _do_user_func {
 
-      my MCE $self  = $_[0];
-      my $_chunk    = $_[1];
-      my $_chunk_id = $_[2];
+      my ($self, $_chunk, $_chunk_id) = @_;
 
       $self->{_chunk_id} = $_chunk_id;
-
       $_user_func->($self, $_chunk, $_chunk_id);
 
       return;
@@ -315,7 +337,7 @@ END {
 
    sub _do_user_func_init {
 
-      my MCE $self = $_[0];
+      my ($self) = @_;
 
       $_user_func = $self->{user_func};
 
@@ -331,13 +353,12 @@ END {
 
 sub _worker_do {
 
-   my MCE $self = $_[0]; my $_params_ref = $_[1];
+   my ($self, $_params_ref) = @_;
 
    @_ = ();
 
-   die "Private method called" unless (caller)[0]->isa( ref($self) );
-
    ## Set options.
+   $self->{_running}    = 1;
    $self->{_abort_msg}  = $_params_ref->{_abort_msg};
    $self->{_run_mode}   = $_params_ref->{_run_mode};
    $self->{_single_dim} = $_params_ref->{_single_dim};
@@ -358,10 +379,10 @@ sub _worker_do {
    my $_task_name  = $self->{task_name};
 
    ## Do not override params if defined in user_tasks during instantiation.
-   for (qw(bounds_only chunk_size interval sequence user_args)) {
-      if (defined $_params_ref->{"_$_"}) {
-         $self->{$_} = $_params_ref->{"_$_"}
-            unless (defined $self->{_task}->{$_});
+   for my $_p (qw(bounds_only chunk_size interval sequence user_args)) {
+      if (defined $_params_ref->{"_${_p}"}) {
+         $self->{$_p} = $_params_ref->{"_${_p}"}
+            unless (defined $self->{_task}->{$_p});
       }
    }
 
@@ -383,8 +404,9 @@ sub _worker_do {
    }
 
    ## Call user_begin if defined.
-   $self->{user_begin}($self, $_task_id, $_task_name)
-      if (defined $self->{user_begin});
+   if (defined $self->{user_begin}) {
+      $self->{user_begin}($self, $_task_id, $_task_name);
+   }
 
    ## Call worker function.
    if ($_run_mode eq 'sequence') {
@@ -432,19 +454,27 @@ sub _worker_do {
    undef $self->{user_data} if (defined $self->{user_data});
 
    ## Call user_end if defined.
-   $self->{user_end}($self, $_task_id, $_task_name)
-      if (defined $self->{user_end});
-
-   $_die_msg = undef;
+   if (defined $self->{user_end}) {
+      $self->{user_end}($self, $_task_id, $_task_name);
+   }
 
    ## Notify the main process a worker has completed.
    local $\ = undef if (defined $\);
 
    flock $_DAT_LOCK, LOCK_EX if ($_lock_chn);
-   print $_DAT_W_SOCK OUTPUT_W_DNE . $LF . $_chn . $LF;
-   print $_DAU_W_SOCK $_task_id . $LF;
+
+   if (exists $self->{_rla_return}) {
+      print {$_DAT_W_SOCK} OUTPUT_W_RLA . $LF . $_chn . $LF;
+      print {$_DAU_W_SOCK} (delete $self->{_rla_return}) . $LF;
+   }
+
+   print {$_DAT_W_SOCK} OUTPUT_W_DNE . $LF . $_chn . $LF;
+   print {$_DAU_W_SOCK} $_task_id . $LF;
+
    flock $_DAT_LOCK, LOCK_UN if ($_lock_chn);
 
+   $self->{_running} = 0;
+
    return;
 }
 
@@ -456,13 +486,11 @@ sub _worker_do {
 
 sub _worker_loop {
 
-   my MCE $self = $_[0];
+   my ($self) = @_;
 
    @_ = ();
 
-   die "Private method called" unless (caller)[0]->isa( ref($self) );
-
-   my ($_response, $_len, $_buffer, $_params_ref);
+   my ($_response, $_len, $_buf, $_params_ref);
 
    my $_COM_LOCK   = $self->{_com_lock};
    my $_COM_W_SOCK = $self->{_com_w_sock};
@@ -477,7 +505,7 @@ sub _worker_loop {
 
          ## Wait until next job request.
          $_response = <$_COM_W_SOCK>;
-         print $_COM_W_SOCK $_wid . $LF;
+         print {$_COM_W_SOCK} $_wid . $LF;
 
          last unless (defined $_response);
          chomp $_response;
@@ -488,16 +516,17 @@ sub _worker_loop {
          if ($_response eq '_data') {
             ## Acquire and process user data.
             chomp($_len = <$_COM_W_SOCK>);
-            read $_COM_W_SOCK, $_buffer, $_len;
+            read $_COM_W_SOCK, $_buf, $_len;
 
-            print $_COM_W_SOCK $_wid . $LF;
+            print {$_COM_W_SOCK} $_wid . $LF;
             flock $_COM_LOCK, LOCK_UN;
 
-            $self->{user_data} = $self->{thaw}($_buffer);
-            undef $_buffer;
+            $self->{user_data} = $self->{thaw}($_buf);
+            undef $_buf;
 
-            select(undef, undef, undef, $_job_delay * $_wid)
-               if (defined $_job_delay && $_job_delay > 0.0);
+            if (defined $_job_delay && $_job_delay > 0.0) {
+               sleep $_job_delay * $_wid;
+            }
 
             _worker_do($self, { });
          }
@@ -505,18 +534,18 @@ sub _worker_loop {
             ## Return to caller if instructed to exit.
             if ($_response eq '_exit') {
                flock $_COM_LOCK, LOCK_UN;
-               return 0;
+               return;
             }
 
             ## Retrieve params data.
             chomp($_len = <$_COM_W_SOCK>);
-            read $_COM_W_SOCK, $_buffer, $_len;
+            read $_COM_W_SOCK, $_buf, $_len;
 
-            print $_COM_W_SOCK $_wid . $LF;
+            print {$_COM_W_SOCK} $_wid . $LF;
             flock $_COM_LOCK, LOCK_UN;
 
-            $_params_ref = $self->{thaw}($_buffer);
-            undef $_buffer;
+            $_params_ref = $self->{thaw}($_buf);
+            undef $_buf;
          }
       }
 
@@ -524,21 +553,23 @@ sub _worker_loop {
       next if ($_response eq '_data');
 
       ## Wait until MCE completes params submission to all workers.
-      my $_c; sysread $self->{_bse_r_sock}, $_c, 1;
+      sysread $self->{_bse_r_sock}, (my $_c), 1;
 
-      select(undef, undef, undef, $_job_delay * $_wid)
-         if (defined $_job_delay && $_job_delay > 0.0);
+      if (defined $_job_delay && $_job_delay > 0.0) {
+         sleep $_job_delay * $_wid;
+      }
 
-      _worker_do($self, $_params_ref); undef $_params_ref;
+      _worker_do($self, $_params_ref);
+
+      undef $_params_ref;
    }
 
    ## Notify the main process a worker has ended. The following is executed
    ## when an invalid reply was received above (not likely to occur).
 
    flock $_COM_LOCK, LOCK_UN;
-   die "worker $self->{_wid} has ended prematurely";
 
-   return 1;
+   die "worker $self->{_wid} has ended prematurely";
 }
 
 ###############################################################################
@@ -549,45 +580,56 @@ sub _worker_loop {
 
 sub _worker_main {
 
-   my MCE $self = $_[0]; my $_wid      = $_[1]; my $_task   = $_[2];
-   my $_task_id = $_[3]; my $_task_wid = $_[4]; my $_params = $_[5];
-
-   my $_plugin_worker_init = $_[6];
+   my ( $self, $_wid, $_task, $_task_id, $_task_wid, $_params,
+        $_plugin_worker_init, $_is_winenv ) = @_;
 
    @_ = ();
 
-   ## Commented out -- fails with the 'forks' module under FreeBSD.
-   ## die "Private method called" unless (caller)[0]->isa( ref($self) );
-
    if (exists $self->{input_data}) {
       my $_ref = ref $self->{input_data};
       delete $self->{input_data} if ($_ref && $_ref ne 'SCALAR');
    }
 
-   ## Define status ID.
+   $self->{_task_id}  = (defined $_task_id ) ? $_task_id  : 0;
+   $self->{_task_wid} = (defined $_task_wid) ? $_task_wid : $_wid;
+   $self->{_task}     = $_task;
+   $self->{_wid}      = $_wid;
+   $self->{_running}  = 0;
+
+   ## Define exit pid and DIE handler.
    my $_use_threads = (defined $_task->{use_threads})
       ? $_task->{use_threads} : $self->{use_threads};
 
-   if ($MCE::_has_threads && $_use_threads) {
-      $self->{_exit_pid} = "TID_" . threads->tid();
+   if ($INC{'threads.pm'} && $_use_threads) {
+      $self->{_exit_pid} = 'TID_' . threads->tid();
    } else {
-      $self->{_exit_pid} = "PID_" . $$;
+      $self->{_exit_pid} = 'PID_' . $$;
    }
 
-   ## Define handlers.
-   $SIG{PIPE} = \&MCE::_NOOP;
+   local $SIG{__DIE__} = sub {
+      if (!defined $^S || $^S) {
+         if ( ($INC{'threads.pm'} && threads->tid() != 0) ||
+               $ENV{'PERL_IPERL_RUNNING'}
+         ) {
+            # thread env or running inside IPerl, check stack trace
+            my $_t = Carp::longmess(); $_t =~ s/\teval [^\n]+\n$//;
 
-   $SIG{__DIE__} = sub {
-      unless ($MCE::_has_threads && $_use_threads) {
-         $_die_msg = (defined $_[0]) ? $_[0] : '';
-         CORE::die(@_);
-      }
-      else {
-         CORE::die(@_) unless (defined $^S);
-         local $SIG{__DIE__} = sub { };
-         local $\ = undef; print STDERR $_[0];
-         $self->exit(255, $_[0]);
+            if ( $_t =~ /^(?:[^\n]+\n){1,7}\teval / ||
+                 $_t =~ /\n\teval [^\n]+\n\t(?:eval|Try)/ )
+            {
+               CORE::die(@_);
+            }
+         }
+         else {
+            # normal env, trust $^S
+            CORE::die(@_);
+         }
       }
+
+      my $_die_msg = (defined $_[0]) ? $_[0] : '';
+      local $SIG{__DIE__} = sub { }; local $\ = undef;
+      print {*STDERR} $_die_msg;
+      $self->exit(255, $_die_msg);
    };
 
    ## Use options from user_tasks if defined.
@@ -603,33 +645,29 @@ sub _worker_main {
    $self->{user_end}    = $_task->{user_end}    if ($_task->{user_end});
 
    ## Init runtime vars. Obtain handle to lock files.
+   my ($_COM_LOCK, $_DAT_LOCK, $_chn);
    my $_mce_sid  = $self->{_mce_sid};
    my $_sess_dir = $self->{_sess_dir};
 
    if (defined $_params && exists $_params->{_chn}) {
-      $self->{_chn} = $_params->{_chn}; delete $_params->{_chn};
+      $_chn = $self->{_chn} = delete $_params->{_chn};
    } else {
-      $self->{_chn} = $_wid % $self->{_data_channels} + 1;
+      $_chn = $self->{_chn} = $_wid % $self->{_data_channels} + 1;
    }
 
-   $self->{_task_id}    = (defined $_task_id ) ? $_task_id  : 0;
-   $self->{_task_wid}   = (defined $_task_wid) ? $_task_wid : $_wid;
-   $self->{_task}       = $_task;
-   $self->{_wid}        = $_wid;
-
-   my ($_COM_LOCK, $_DAT_LOCK);
-   my $_lock_chn = $self->{_lock_chn};
-   my $_chn      = $self->{_chn};
-
-   for (1 .. $self->{_data_channels}) {
-      $self->{_dat_r_sock}->[$_] = $self->{_dat_w_sock}->[$_] = undef
-         unless ($_ == $_chn);
+   ## Unset the need for channel locking if only worker on the channel.
+   if ($self->{_init_total_workers} < DATA_CHANNELS * 2) {
+      if ($_wid > $self->{_init_total_workers} % DATA_CHANNELS) {
+         $self->{_lock_chn} = 0 if ($_wid <= DATA_CHANNELS);
+      }
    }
 
-   if ($_lock_chn) {
+   ## Choose locks for DATA channels.
+   if ($self->{_lock_chn}) {
       open $_DAT_LOCK, '+>>:raw:stdio', "$_sess_dir/_dat.lock.$_chn"
          or die "(W) open error $_sess_dir/_dat.lock.$_chn: $!\n";
    }
+
    open $_COM_LOCK, '+>>:raw:stdio', "$_sess_dir/_com.lock"
       or die "(W) open error $_sess_dir/_com.lock: $!\n";
 
@@ -641,52 +679,44 @@ sub _worker_main {
       flush_file flush_stderr flush_stdout stderr_file stdout_file
       on_post_exit on_post_run user_data user_error user_output
       _pids _state _status _thrs _tids
-   )};
+   ) };
 
-   foreach (keys %MCE::_mce_spawned) {
-      delete $MCE::_mce_spawned{$_} unless ($_ eq $_mce_sid);
-   }
+   MCE::_clean_sessions($_mce_sid);
 
    ## Call module's worker_init routine for modules plugged into MCE.
-   $_->($self) for (@{ $_plugin_worker_init });
+   for my $_p (@{ $_plugin_worker_init }) { $_p->($self); }
 
    _do_send_init($self);
 
    ## Begin processing if worker was added during processing. Otherwise,
    ## respond back to the main process if the last worker spawned.
    if (defined $_params) {
-      select(undef, undef, undef, 0.002);
-      _worker_do($self, $_params); undef $_params;
-   }
-   elsif ($self->{_wid} == $self->{_total_workers}) {
-      my $_buffer; my $_COM_W_SOCK = $self->{_com_w_sock};
-      sysread $self->{_que_r_sock}, $_buffer, 1;
-      local $\ = undef; print $_COM_W_SOCK $LF;
+      sleep 0.002; _worker_do($self, $_params); undef $_params;
+   } else {
+      lock $MCE::_WIN_LOCK if ($_is_winenv);
    }
 
-   ## Enter worker loop.
-   my $_status = _worker_loop($self);
+   ## Enter worker loop. Clear worker session after running.
+   _worker_loop($self); _do_send_clear($self);
 
-   delete $MCE::_mce_spawned{$_mce_sid};
+   $self->{_com_lock} = undef;
+   $self->{_dat_lock} = undef;
+
+   MCE::_clear_session($_mce_sid);
 
    ## Wait until MCE completes exit notification.
    $SIG{__DIE__} = $SIG{__WARN__} = sub { };
 
-   eval {
-      my $_c; sysread $self->{_bse_r_sock}, $_c, 1;
+   local $@; eval {
+      sysread $self->{_bse_r_sock}, (my $_c), 1;
    };
 
-   select(undef, undef, undef, 0.005) if ($MCE::_is_WinEnv);
-
-   if ($_lock_chn) {
+   if ($self->{_lock_chn}) {
       close $_DAT_LOCK; undef $_DAT_LOCK;
    }
 
    close $_COM_LOCK; undef $_COM_LOCK;
 
-   select STDERR; $| = 1;
-   select STDOUT; $| = 1;
-
    return;
 }
 
