@@ -9,12 +9,17 @@ package MCE::Grep;
 use strict;
 use warnings;
 
+## no critic (BuiltinFunctions::ProhibitStringyEval)
+## no critic (Subroutines::ProhibitSubroutinePrototypes)
+## no critic (TestingAndDebugging::ProhibitNoStrict)
+
 use Scalar::Util qw( looks_like_number );
 
 use MCE;
-use MCE::Util;
 
-our $VERSION = '1.517'; $VERSION = eval $VERSION;
+our $VERSION  = '1.608';
+
+our @CARP_NOT = qw( MCE );
 
 ###############################################################################
 ## ----------------------------------------------------------------------------
@@ -22,8 +27,8 @@ our $VERSION = '1.517'; $VERSION = eval $VERSION;
 ##
 ###############################################################################
 
-our $MAX_WORKERS = 'auto';
-our $CHUNK_SIZE  = 'auto';
+my $MAX_WORKERS = 'auto';
+my $CHUNK_SIZE  = 'auto';
 
 my ($_MCE, $_loaded); my ($_params, $_prev_c); my $_tag = 'MCE::Grep';
 
@@ -32,26 +37,38 @@ sub import {
    my $_class = shift; return if ($_loaded++);
 
    ## Process module arguments.
-   while (my $_arg = shift) {
+   while (my $_argument = shift) {
+      my $_arg = lc $_argument;
 
-      $MAX_WORKERS  = shift and next if ( $_arg =~ /^max_workers$/i );
-      $CHUNK_SIZE   = shift and next if ( $_arg =~ /^chunk_size$/i );
-      $MCE::TMP_DIR = shift and next if ( $_arg =~ /^tmp_dir$/i );
-      $MCE::FREEZE  = shift and next if ( $_arg =~ /^freeze$/i );
-      $MCE::THAW    = shift and next if ( $_arg =~ /^thaw$/i );
+      $MAX_WORKERS = shift and next if ( $_arg eq 'max_workers' );
+      $CHUNK_SIZE  = shift and next if ( $_arg eq 'chunk_size' );
 
-      if ( $_arg =~ /^sereal$/i ) {
+      $MCE::FREEZE = $MCE::MCE->{freeze} = shift and next
+         if ( $_arg eq 'freeze' );
+      $MCE::THAW = $MCE::MCE->{thaw} = shift and next
+         if ( $_arg eq 'thaw' );
+
+      if ( $_arg eq 'sereal' ) {
          if (shift eq '1') {
             local $@; eval 'use Sereal qw(encode_sereal decode_sereal)';
             unless ($@) {
-               $MCE::FREEZE = \&encode_sereal;
-               $MCE::THAW   = \&decode_sereal;
+               $MCE::FREEZE = $MCE::MCE->{freeze} = \&encode_sereal;
+               $MCE::THAW = $MCE::MCE->{thaw} = \&decode_sereal;
             }
          }
          next;
       }
 
-      _croak("$_tag::import: '$_arg' is not a valid module argument");
+      if ( $_arg eq 'tmp_dir' ) {
+         $MCE::TMP_DIR = $MCE::MCE->{tmp_dir} = shift;
+         my $_e1 = 'is not a directory or does not exist';
+         my $_e2 = 'is not writeable';
+         _croak($_tag."::import: ($MCE::TMP_DIR) $_e1") unless -d $MCE::TMP_DIR;
+         _croak($_tag."::import: ($MCE::TMP_DIR) $_e2") unless -w $MCE::TMP_DIR;
+         next;
+      }
+
+      _croak($_tag."::import: ($_argument) is not a valid module argument");
    }
 
    $MAX_WORKERS = MCE::Util::_parse_max_workers($MAX_WORKERS);
@@ -62,11 +79,11 @@ sub import {
 
    ## Import functions.
    no strict 'refs'; no warnings 'redefine';
-   my $_package = caller();
+   my $_pkg = caller;
 
-   *{ $_package . '::mce_grep_f' } = \&mce_grep_f;
-   *{ $_package . '::mce_grep_s' } = \&mce_grep_s;
-   *{ $_package . '::mce_grep'   } = \&mce_grep;
+   *{ $_pkg.'::mce_grep_f' } = \&run_file;
+   *{ $_pkg.'::mce_grep_s' } = \&run_seq;
+   *{ $_pkg.'::mce_grep'   } = \&run;
 
    return;
 }
@@ -74,7 +91,7 @@ sub import {
 END {
    return if (defined $_MCE && $_MCE->wid);
 
-   MCE::Grep::finish();
+   finish();
 }
 
 ###############################################################################
@@ -87,7 +104,9 @@ my ($_total_chunks, %_tmp);
 
 sub _gather {
 
-   $_tmp{$_[1]} = $_[0];
+   my ($_chunk_id, $_data_ref) = @_;
+
+   $_tmp{$_chunk_id} = $_data_ref;
    $_total_chunks++;
 
    return;
@@ -101,23 +120,24 @@ sub _gather {
 
 sub init (@) {
 
+   shift if (defined $_[0] && $_[0] eq 'MCE::Grep');
+
    if (MCE->wid) {
       @_ = (); _croak(
          "$_tag: function cannot be called by the worker process"
       );
    }
 
-   _croak("$_tag: 'argument' is not a HASH reference")
-      unless (ref $_[0] eq 'HASH');
+   finish(); $_params = (ref $_[0] eq 'HASH') ? shift : { @_ };
 
-   MCE::Grep::finish(); $_params = shift;
+   @_ = ();
 
    return;
 }
 
 sub finish () {
 
-   if (defined $_MCE) {
+   if (defined $_MCE && $_MCE->{_spawned}) {
       MCE::_save_state; $_MCE->shutdown(); MCE::_restore_state;
    }
 
@@ -132,7 +152,9 @@ sub finish () {
 ##
 ###############################################################################
 
-sub mce_grep_f (&@) {
+sub run_file (&@) {
+
+   shift if (defined $_[0] && $_[0] eq 'MCE::Grep');
 
    my $_code = shift; my $_file = shift;
 
@@ -144,22 +166,22 @@ sub mce_grep_f (&@) {
       $_params = {};
    }
 
-   if (defined $_file && ref $_file eq "" && $_file ne "") {
-      _croak("$_tag: '$_file' does not exist") unless (-e $_file);
-      _croak("$_tag: '$_file' is not readable") unless (-r $_file);
-      _croak("$_tag: '$_file' is not a plain file") unless (-f $_file);
+   if (defined $_file && ref $_file eq '' && $_file ne '') {
+      _croak("$_tag: ($_file) does not exist") unless (-e $_file);
+      _croak("$_tag: ($_file) is not readable") unless (-r $_file);
+      _croak("$_tag: ($_file) is not a plain file") unless (-f $_file);
       $_params->{_file} = $_file;
    }
    elsif (ref $_file eq 'GLOB' || ref $_file eq 'SCALAR' || ref($_file) =~ /^IO::/) {
       $_params->{_file} = $_file;
    }
    else {
-      _croak("$_tag: 'file' is not specified or valid");
+      _croak("$_tag: (file) is not specified or valid");
    }
 
    @_ = ();
 
-   return mce_grep($_code);
+   return run($_code);
 }
 
 ###############################################################################
@@ -168,7 +190,9 @@ sub mce_grep_f (&@) {
 ##
 ###############################################################################
 
-sub mce_grep_s (&@) {
+sub run_seq (&@) {
+
+   shift if (defined $_[0] && $_[0] eq 'MCE::Grep');
 
    my $_code = shift;
 
@@ -190,23 +214,25 @@ sub mce_grep_s (&@) {
       $_begin = $_[0]->[0]; $_end = $_[0]->[1];
       $_params->{sequence} = $_[0];
    }
-   elsif (ref $_[0] eq "") {
+   elsif (ref $_[0] eq '') {
       $_begin = $_[0]; $_end = $_[1];
       $_params->{sequence} = [ @_ ];
    }
    else {
-      _croak("$_tag: 'sequence' is not specified or valid");
+      _croak("$_tag: (sequence) is not specified or valid");
    }
 
-   _croak("$_tag: 'begin' is not specified for sequence")
+   _croak("$_tag: (begin) is not specified for sequence")
       unless (defined $_begin);
 
-   _croak("$_tag: 'end' is not specified for sequence")
+   _croak("$_tag: (end) is not specified for sequence")
       unless (defined $_end);
 
+   $_params->{sequence_run} = 1;
+
    @_ = ();
 
-   return mce_grep($_code);
+   return run($_code);
 }
 
 ###############################################################################
@@ -215,7 +241,9 @@ sub mce_grep_s (&@) {
 ##
 ###############################################################################
 
-sub mce_grep (&@) {
+sub run (&@) {
+
+   shift if (defined $_[0] && $_[0] eq 'MCE::Grep');
 
    my $_code = shift;   $_total_chunks = 0; undef %_tmp;
 
@@ -248,10 +276,11 @@ sub mce_grep (&@) {
    );
 
    if (defined $_params) {
-      $_input_data = $_params->{input_data} if (exists $_params->{input_data});
-
       if (exists $_params->{_file}) {
-         $_input_data = $_params->{_file}; delete $_params->{_file};
+         $_input_data = delete $_params->{_file};
+      }
+      else {
+         $_input_data = $_params->{input_data} if exists $_params->{input_data};
       }
    }
 
@@ -264,7 +293,7 @@ sub mce_grep (&@) {
       $_prev_c = $_code;
 
       my %_options = (
-         use_threads => 0, max_workers => $_max_workers, task_name => $_tag,
+         max_workers => $_max_workers, task_name => $_tag,
          user_func => sub {
 
             my ($_mce, $_chunk_ref, $_chunk_id) = @_;
@@ -276,18 +305,18 @@ sub mce_grep (&@) {
                if (ref $_chunk_ref eq 'SCALAR') {
                   local $/ = $_mce->{RS} if defined $_mce->{RS};
                   open my  $_MEM_FH, '<', $_chunk_ref;
-                  while ( <$_MEM_FH> ) { push (@_a, $_) if &$_code; }
+                  while ( <$_MEM_FH> ) { push (@_a, $_) if &{ $_code }; }
                   close    $_MEM_FH;
                }
                else {
                   if (ref $_chunk_ref) {
-                     push @_a, grep { &$_code } @$_chunk_ref;
+                     push @_a, grep { &{ $_code } } @{ $_chunk_ref };
                   } else {
-                     push @_a, grep { &$_code } $_chunk_ref;
+                     push @_a, grep { &{ $_code } } $_chunk_ref;
                   }
                }
 
-               MCE->gather(\@_a, $_chunk_id);
+               MCE->gather($_chunk_id, \@_a);
             }
             else {
                my $_cnt = 0;
@@ -295,31 +324,32 @@ sub mce_grep (&@) {
                if (ref $_chunk_ref eq 'SCALAR') {
                   local $/ = $_mce->{RS} if defined $_mce->{RS};
                   open my  $_MEM_FH, '<', $_chunk_ref;
-                  while ( <$_MEM_FH> ) { $_cnt++ if &$_code; }
+                  while ( <$_MEM_FH> ) { $_cnt++ if &{ $_code }; }
                   close    $_MEM_FH;
                }
                else {
                   if (ref $_chunk_ref) {
-                     $_cnt += grep { &$_code } @$_chunk_ref;
+                     $_cnt += grep { &{ $_code } } @{ $_chunk_ref };
                   } else {
-                     $_cnt += grep { &$_code } $_chunk_ref;
+                     $_cnt += grep { &{ $_code } } $_chunk_ref;
                   }
                }
 
                MCE->gather($_cnt) if defined $_wantarray;
             }
-         }
+         },
       );
 
       if (defined $_params) {
-         foreach (keys %{ $_params }) {
-            next if ($_ eq 'input_data');
-            next if ($_ eq 'chunk_size');
+         for my $_p (keys %{ $_params }) {
+            next if ($_p eq 'sequence_run');
+            next if ($_p eq 'input_data');
+            next if ($_p eq 'chunk_size');
 
-            _croak("MCE::Grep: '$_' is not a valid constructor argument")
-               unless (exists $MCE::_valid_fields_new{$_});
+            _croak("MCE::Grep: ($_p) is not a valid constructor argument")
+               unless (exists $MCE::_valid_fields_new{$_p});
 
-            $_options{$_} = $_params->{$_};
+            $_options{$_p} = $_params->{$_p};
          }
       }
 
@@ -337,18 +367,35 @@ sub mce_grep (&@) {
       ? \&_gather : sub { $_cnt += $_[0]; return; };
 
    if (defined $_input_data) {
-      @_ = (); $_MCE->process({ chunk_size => $_chunk_size }, $_input_data);
+      @_ = ();
+      $_MCE->process({ chunk_size => $_chunk_size }, $_input_data);
+      delete $_MCE->{input_data};
    }
    elsif (scalar @_) {
       $_MCE->process({ chunk_size => $_chunk_size }, \@_);
+      delete $_MCE->{input_data};
    }
    else {
-      $_MCE->run({ chunk_size => $_chunk_size }, 0)
-         if (defined $_params && exists $_params->{sequence});
+      if (defined $_params && exists $_params->{sequence}) {
+         $_MCE->run({
+            chunk_size => $_chunk_size, sequence => $_params->{sequence}
+         }, 0);
+         if (exists $_params->{sequence_run}) {
+            delete $_params->{sequence_run};
+            delete $_params->{sequence};
+         }
+         delete $_MCE->{sequence};
+      }
    }
 
    MCE::_restore_state;
 
+   if (exists $_MCE->{_rla_return}) {
+      $MCE::MCE->{_rla_return} = delete $_MCE->{_rla_return};
+   }
+
+   finish() if ($^S);   ## shutdown if in eval state
+
    if ($_wantarray) {
       return map { @{ $_ } } delete @_tmp{ 1 .. $_total_chunks };
    }
@@ -372,12 +419,15 @@ sub _croak {
 
 sub _validate_number {
 
-   my $_n = $_[0]; my $_key = $_[1];
+   my ($_n, $_key) = @_;
+
+   _croak("$_tag: ($_key) is not valid") if (!defined $_n);
 
    $_n =~ s/K\z//i; $_n =~ s/M\z//i;
 
-   _croak("$_tag: '$_key' is not valid")
-      if (!looks_like_number($_n) || int($_n) != $_n || $_n < 1);
+   if (!looks_like_number($_n) || int($_n) != $_n || $_n < 1) {
+      _croak("$_tag: ($_key) is not valid");
+   }
 
    return;
 }
@@ -398,7 +448,7 @@ MCE::Grep - Parallel grep model similar to the native grep function
 
 =head1 VERSION
 
-This document describes MCE::Grep version 1.517
+This document describes MCE::Grep version 1.608
 
 =head1 SYNOPSIS
 
@@ -410,9 +460,9 @@ This document describes MCE::Grep version 1.517
    my @b = mce_grep { $_ % 5 == 0 } [ 1..10000 ];
 
    ## File_path, glob_ref, or scalar_ref
-   my @c = mce_grep_f { /phrase/ } "/path/to/file";
-   my @d = mce_grep_f { /phrase/ } $file_handle;
-   my @e = mce_grep_f { /phrase/ } \$scalar;
+   my @c = mce_grep_f { /pattern/ } "/path/to/file";
+   my @d = mce_grep_f { /pattern/ } $file_handle;
+   my @e = mce_grep_f { /pattern/ } \$scalar;
 
    ## Sequence of numbers (begin, end [, step, format])
    my @f = mce_grep_s { %_ * 3 == 0 } 1, 10000, 5;
@@ -424,116 +474,107 @@ This document describes MCE::Grep version 1.517
 
 =head1 DESCRIPTION
 
-This module provides a parallel grep implementation via Many-core Engine. MCE
-incurs a small overhead due to passing of data. Therefore, a fast code block
-will likely run faster using the native grep function in Perl. The overhead
-quickly diminishes as the complexity of the code block increases.
+This module provides a parallel grep implementation via Many-Core Engine.
+MCE incurs a small overhead due to passing of data. A fast code block will
+run faster natively. However, the overhead will likely diminish as the
+complexity increases for the code.
 
-   my @m1 =     grep { $_ % 5 == 0 } 1..1000000;          ## 0.137 secs
-   my @m2 = mce_grep { $_ % 5 == 0 } 1..1000000;          ## 0.295 secs
+   my @m1 =     grep { $_ % 5 == 0 } 1..1000000;          ## 0.065 secs
+   my @m2 = mce_grep { $_ % 5 == 0 } 1..1000000;          ## 0.194 secs
 
 Chunking, enabled by default, greatly reduces the overhead behind the scene.
 The time for mce_grep below also includes the time for data exchanges between
 the manager and worker processes. More parallelization will be seen when the
-code block requires additional CPU time code-wise.
+code incurs additional CPU time.
 
-   my @m1 =     grep { /[2357][1468][9]/ } 1..1000000;    ## 0.653 secs
-   my @m2 = mce_grep { /[2357][1468][9]/ } 1..1000000;    ## 0.347 secs
+   my @m1 =     grep { /[2357][1468][9]/ } 1..1000000;    ## 0.353 secs
+   my @m2 = mce_grep { /[2357][1468][9]/ } 1..1000000;    ## 0.218 secs
 
-The mce_grep_s function will provide better times, useful when the input data
-is simply a range of numbers. Workers generate sequences mathematically among
-themselves without any interaction from the manager process. Two arguments are
-required for mce_grep_s (begin, end). Step defaults to 1 if begin is smaller
-than end, otherwise -1.
+Even faster is mce_grep_s; useful when input data is a range of numbers.
+Workers generate sequences mathematically among themselves without any
+interaction from the manager process. Two arguments are required for
+mce_grep_s (begin, end). Step defaults to 1 if begin is smaller than end,
+otherwise -1.
 
-   my @m3 = mce_grep_s { /[2357][1468][9]/ } 1, 1000000;  ## 0.271 secs
+   my @m3 = mce_grep_s { /[2357][1468][9]/ } 1, 1000000;  ## 0.165 secs
 
-Although this document is about MCE::Grep, the L<MCE::Stream> module can write
-results immediately without waiting for all chunks to complete. This is made
-possible by passing the reference of the array (in this case @m4 and @m5).
+Although this document is about MCE::Grep, the L<MCE::Stream|MCE::Stream>
+module can write results immediately without waiting for all chunks to
+complete. This is made possible by passing the reference to an array
+(in this case @m4 and @m5).
 
    use MCE::Stream default_mode => 'grep';
 
    my @m4; mce_stream \@m4, sub { /[2357][1468][9]/ }, 1..1000000;
 
-      ## Completed in 0.304 secs. That is amazing considering the
-      ## overhead for passing data between the manager and worker.
+      ## Completed in 0.203 secs. This is amazing considering the
+      ## overhead for passing data between the manager and workers.
 
    my @m5; mce_stream_s \@m5, sub { /[2357][1468][9]/ }, 1, 1000000;
 
-      ## Completed in 0.227 secs. Like with mce_grep_s, specifying a
+      ## Completed in 0.120 secs. Like with mce_grep_s, specifying a
       ## sequence specification turns out to be faster due to lesser
       ## overhead for the manager process.
 
-A good use-case for MCE::Grep is for searching through a large log file much
-like one might do using the native grep function. Lets assume the file contains
-a hundred thousand records separated by a string "::\n\n" between each record.
-The imaginary pattern used also returns less than 50 records.
-
-The native implementation is what one might do actually. What's not clearly
-visible here is the initial memory consumption, due to Perl reading the entire
-content into memory, prior to grep actually starting. A 300 MB file will
-consume roughly 640 MB. The time to run is 1.217 seconds for the file
-residing in the OS-level file-system cache.
-
-   $/ = "::\n\n";
-
-   open my $LOG, "<", "/path/to/log/file";
-   my @match = grep { $_ =~ /pattern/ } <$LOG>;
-   close $LOG;
-
-The memory utilization is much better with MCE; 8 workers * 23 MB = 184 MB.
-MCE caps at some point, therefore allowing one to process a file much larger
-than available memory. The time to run is 0.416 seconds (2.93x faster) which
-includes the overhead for chunking and serializing data back to the main
-process as if processing serially.
+A common scenario is grepping for pattern(s) inside a massive log file.
+Notice how parallelism increases as complexity increases for the pattern.
+Testing was done against a 300 MB file containing 250k lines.
 
    use MCE::Grep;
 
-   MCE::Grep::init { RS => "::\n\n" };
+   my @m; open my $LOG, "<", "/path/to/log/file" or die "$!\n";
 
-   my @match = mce_grep_f { $_ =~ /pattern/ } "/path/to/file";
+   @m = grep { /pattern/ } <$LOG>;                      ##  0.756 secs
+   @m = grep { /foobar|[2357][1468][9]/ } <$LOG>;       ## 24.681 secs
 
-It gets even better for counting though. The native grep function takes 1.136
-seconds to run whereas MCE takes just 0.155 seconds (7.33x faster).
+   ## Parallelism with mce_grep. This involves the manager process
+   ## due to processing a file handle.
 
-   ## Native Grep
-   my $count = grep { $_ =~ /pattern/ } <$LOG>;
+   @m = mce_grep { /pattern/ } <$LOG>;                  ##  0.997 secs
+   @m = mce_grep { /foobar|[2357][1468][9]/ } <$LOG>;   ##  7.439 secs
 
-   ## MCE Grep
-   my $count = mce_grep_f { $_ =~ /pattern/ } "/path/to/file";
+   ## Even faster with mce_grep_f. Workers access the file directly
+   ## with zero interaction from the manager process.
+
+   my $LOG = "/path/to/file";
+   @m = mce_grep_f { /pattern/ } $LOG;                  ##  0.112 secs
+   @m = mce_grep_f { /foobar|[2357][1468][9]/ } $LOG;   ##  6.840 secs
 
 =head1 OVERRIDING DEFAULTS
 
 The following list 5 options which may be overridden when loading the module.
 
-   use Sereal qw(encode_sereal decode_sereal);
+   use Sereal qw( encode_sereal decode_sereal );
+   use CBOR::XS qw( encode_cbor decode_cbor );
+   use JSON::XS qw( encode_json decode_json );
 
    use MCE::Grep
-         max_workers => 4,                    ## Default 'auto' 
-         chunk_size  => 100,                  ## Default 'auto'
-         tmp_dir     => "/path/to/app/tmp",   ## $MCE::Signal::tmp_dir
-         freeze      => \&encode_sereal,      ## \&Storable::freeze
-         thaw        => \&decode_sereal       ## \&Storable::thaw
+         max_workers => 4,               ## Default 'auto'
+         chunk_size => 100,              ## Default 'auto'
+         tmp_dir => "/path/to/app/tmp",  ## $MCE::Signal::tmp_dir
+         freeze => \&encode_sereal,      ## \&Storable::freeze
+         thaw => \&decode_sereal         ## \&Storable::thaw
    ;
 
 There is a simpler way to enable Sereal with MCE 1.5. The following will
-attempt to use Sereal if available, otherwise will default back to using
-Storable for serialization.
+attempt to use Sereal if available, otherwise defaults to Storable for
+serialization.
 
    use MCE::Grep Sereal => 1;
 
-   ## Serialization is through Sereal if available.
+   ## Serialization is by the Sereal module if available.
    my @m2 = mce_grep { $_ % 5 == 0 } 1..10000;
 
 =head1 CUSTOMIZING MCE
 
 =over 3
 
-=item init
+=item MCE::Grep->init ( options )
+
+=item MCE::Grep::init { options }
 
 The init function accepts a hash of MCE options. The gather option, if
-specified, will be set to undef due to being used internally by the module.
+specified, is ignored due to being used internally by the module.
 
    use MCE::Grep;
 
@@ -572,23 +613,38 @@ specified, will be set to undef due to being used internally by the module.
 
 =over 3
 
+=item MCE::Grep->run ( sub { code }, iterator )
+
+=item mce_grep { code } iterator
+
+An iterator reference can by specified for input_data. Iterators are described
+under "SYNTAX for INPUT_DATA" at L<MCE::Core|MCE::Core>.
+
+   my @a = mce_grep { $_ % 3 == 0 } make_iterator(10, 30, 2);
+
+=item MCE::Grep->run ( sub { code }, list )
+
 =item mce_grep { code } list
 
-Input data can be defined using a list or passing a reference to an array.
+Input data can be defined using a list.
 
    my @a = mce_grep { /[2357]/ } 1..1000;
    my @b = mce_grep { /[2357]/ } [ 1..1000 ];
 
+=item MCE::Grep->run_file ( sub { code }, file )
+
 =item mce_grep_f { code } file
 
 The fastest of these is the /path/to/file. Workers communicate the next offset
 position among themselves without any interaction from the manager process.
 
-   my @c = mce_grep_f { /phrase/ } "/path/to/file";
-   my @d = mce_grep_f { /phrase/ } $file_handle;
-   my @e = mce_grep_f { /phrase/ } \$scalar;
+   my @c = mce_grep_f { /pattern/ } "/path/to/file";
+   my @d = mce_grep_f { /pattern/ } $file_handle;
+   my @e = mce_grep_f { /pattern/ } \$scalar;
 
-=item mce_grep_s { code } sequence
+=item MCE::Grep->run_seq ( sub { code }, $beg, $end [, $step, $fmt ] )
+
+=item mce_grep_s { code } $beg, $end [, $step, $fmt ]
 
 Sequence can be defined as a list, an array reference, or a hash reference.
 The functions require both begin and end values to run. Step and format are
@@ -603,24 +659,19 @@ optional. The format is passed to sprintf (% may be omitted below).
       begin => $beg, end => $end, step => $step, format => $fmt
    };
 
-=item mce_grep { code } iterator
-
-An iterator reference can by specified for input data. Iterators are described
-under "SYNTAX for INPUT_DATA" at L<MCE::Core>.
-
-   my @a = mce_grep { $_ % 3 == 0 } make_iterator(10, 30, 2);
-
 =back
 
 =head1 MANUAL SHUTDOWN
 
 =over 3
 
-=item finish
+=item MCE::Grep->finish
 
-MCE workers remain persistent as much as possible after running. Shutdown
-occurs when the script exits. One can manually shutdown MCE by simply calling
-finish after running. This resets the MCE instance.
+=item MCE::Grep::finish
+
+Workers remain persistent as much as possible after running. Shutdown occurs
+automatically when the script terminates. Call finish when workers are no
+longer needed.
 
    use MCE::Grep;
 
@@ -636,19 +687,11 @@ finish after running. This resets the MCE instance.
 
 =head1 INDEX
 
-L<MCE>
+L<MCE|MCE>
 
 =head1 AUTHOR
 
 Mario E. Roy, S<E<lt>marioeroy AT gmail DOT comE<gt>>
 
-=head1 LICENSE
-
-This program is free software; you can redistribute it and/or modify it
-under the terms of either: the GNU General Public License as published
-by the Free Software Foundation; or the Artistic License.
-
-See L<http://dev.perl.org/licenses/> for more information.
-
 =cut
 
