@@ -9,14 +9,18 @@ package MCE::Step;
 use strict;
 use warnings;
 
+## no critic (BuiltinFunctions::ProhibitStringyEval)
+## no critic (Subroutines::ProhibitSubroutinePrototypes)
+## no critic (TestingAndDebugging::ProhibitNoStrict)
+
 use Scalar::Util qw( looks_like_number );
 
 use MCE;
-use MCE::Util;
-
 use MCE::Queue;
 
-our $VERSION = '1.517'; $VERSION = eval $VERSION;
+our $VERSION  = '1.608';
+
+our @CARP_NOT = qw( MCE );
 
 ###############################################################################
 ## ----------------------------------------------------------------------------
@@ -24,8 +28,9 @@ our $VERSION = '1.517'; $VERSION = eval $VERSION;
 ##
 ###############################################################################
 
-our $MAX_WORKERS  = 'auto';
-our $CHUNK_SIZE   = 'auto';
+my $MAX_WORKERS = 'auto';
+my $CHUNK_SIZE  = 'auto';
+my $FAST        = 0;
 
 my ($_params, @_prev_c, @_prev_n, @_prev_w, @_user_tasks, @_queue);
 my ($_MCE, $_loaded, $_last_task_id); my $_tag = 'MCE::Step';
@@ -35,26 +40,43 @@ sub import {
    my $_class = shift; return if ($_loaded++);
 
    ## Process module arguments.
-   while (my $_arg = shift) {
+   while (my $_argument = shift) {
+      my $_arg = lc $_argument;
 
-      $MAX_WORKERS  = shift and next if ( $_arg =~ /^max_workers$/i );
-      $CHUNK_SIZE   = shift and next if ( $_arg =~ /^chunk_size$/i );
-      $MCE::TMP_DIR = shift and next if ( $_arg =~ /^tmp_dir$/i );
-      $MCE::FREEZE  = shift and next if ( $_arg =~ /^freeze$/i );
-      $MCE::THAW    = shift and next if ( $_arg =~ /^thaw$/i );
+      $MAX_WORKERS = shift and next if ( $_arg eq 'max_workers' );
+      $CHUNK_SIZE  = shift and next if ( $_arg eq 'chunk_size' );
 
-      if ( $_arg =~ /^sereal$/i ) {
+      $MCE::FREEZE = $MCE::MCE->{freeze} = shift and next
+         if ( $_arg eq 'freeze' );
+      $MCE::THAW = $MCE::MCE->{thaw} = shift and next
+         if ( $_arg eq 'thaw' );
+
+      if ( $_arg eq 'sereal' ) {
          if (shift eq '1') {
             local $@; eval 'use Sereal qw(encode_sereal decode_sereal)';
             unless ($@) {
-               $MCE::FREEZE = \&encode_sereal;
-               $MCE::THAW   = \&decode_sereal;
+               $MCE::FREEZE = $MCE::MCE->{freeze} = \&encode_sereal;
+               $MCE::THAW = $MCE::MCE->{thaw} = \&decode_sereal;
             }
          }
          next;
       }
 
-      _croak("$_tag::import: '$_arg' is not a valid module argument");
+      if ( $_arg eq 'tmp_dir' ) {
+         $MCE::TMP_DIR = $MCE::MCE->{tmp_dir} = shift;
+         my $_e1 = 'is not a directory or does not exist';
+         my $_e2 = 'is not writeable';
+         _croak($_tag."::import: ($MCE::TMP_DIR) $_e1") unless -d $MCE::TMP_DIR;
+         _croak($_tag."::import: ($MCE::TMP_DIR) $_e2") unless -w $MCE::TMP_DIR;
+         next;
+      }
+
+      if ( $_arg eq 'fast' ) {
+         $FAST = 1 if (shift eq '1');
+         next;
+      }
+
+      _croak($_tag."::import: ($_argument) is not a valid module argument");
    }
 
    $MAX_WORKERS = MCE::Util::_parse_max_workers($MAX_WORKERS);
@@ -65,11 +87,11 @@ sub import {
 
    ## Import functions.
    no strict 'refs'; no warnings 'redefine';
-   my $_package = caller();
+   my $_pkg = caller;
 
-   *{ $_package . '::mce_step_f' } = \&mce_step_f;
-   *{ $_package . '::mce_step_s' } = \&mce_step_s;
-   *{ $_package . '::mce_step'   } = \&mce_step;
+   *{ $_pkg.'::mce_step_f' } = \&run_file;
+   *{ $_pkg.'::mce_step_s' } = \&run_seq;
+   *{ $_pkg.'::mce_step'   } = \&run;
 
    return;
 }
@@ -77,7 +99,7 @@ sub import {
 END {
    return if (defined $_MCE && $_MCE->wid);
 
-   MCE::Step::finish();
+   finish();
 }
 
 ###############################################################################
@@ -92,8 +114,8 @@ sub _task_end {
    my ($_mce, $_task_id, $_task_name) = @_;
 
    if (defined $_mce->{user_tasks}->[$_task_id + 1]) {
-      my $N_workers = $_mce->{user_tasks}->[$_task_id + 1]->{max_workers};
-      $_queue[$_task_id]->enqueue((undef) x $N_workers);
+      my $n_workers = $_mce->{user_tasks}->[$_task_id + 1]->{max_workers};
+      $_queue[$_task_id]->enqueue((undef) x $n_workers);
    }
 
    $_params->{task_end}->($_mce, $_task_id, $_task_name)
@@ -107,18 +129,22 @@ sub _task_end {
 
    sub MCE::step {
 
-      my $x = shift; my MCE $self = ref($x) ? $x : $_MCE;
+      my $x = shift; my $self = ref($x) ? $x : $_MCE;
 
-      _croak("MCE::step: method cannot be called by the manager process")
+      _croak('MCE::step: method cannot be called by the manager process')
          unless ($self->{_wid});
 
       my $_task_id = $self->{_task_id};
 
       if ($_task_id < $_last_task_id) {
-         $_queue[$_task_id]->enqueue($self->freeze([ @_ ]));
+         if (scalar @_ > 1 || ref $_[0] || !defined $_[0]) {
+            $_queue[$_task_id]->enqueue($self->freeze([ @_ ]).'1');
+         } else {
+            $_queue[$_task_id]->enqueue($_[0].'0');
+         }
       }
       else {
-         _croak("MCE::step: method cannot be called by the last task");
+         _croak('MCE::step: method cannot be called by the last task');
       }
 
       return;
@@ -133,29 +159,30 @@ sub _task_end {
 
 sub init (@) {
 
+   shift if (defined $_[0] && $_[0] eq 'MCE::Step');
+
    if (MCE->wid) {
       @_ = (); _croak(
          "$_tag: function cannot be called by the worker process"
       );
    }
 
-   _croak("$_tag: 'argument' is not a HASH reference")
-      unless (ref $_[0] eq 'HASH');
+   finish(); $_params = (ref $_[0] eq 'HASH') ? shift : { @_ };
 
-   MCE::Step::finish(); $_params = shift;
+   @_ = ();
 
    return;
 }
 
 sub finish () {
 
-   if (defined $_MCE) {
+   if (defined $_MCE && $_MCE->{_spawned}) {
       MCE::_save_state; $_MCE->shutdown(); MCE::_restore_state;
    }
 
    @_user_tasks = (); @_prev_w = (); @_prev_n = (); @_prev_c = ();
 
-   $_->DESTROY() foreach (@_queue); @_queue = ();
+   $_->DESTROY() for (@_queue); @_queue = ();
 
    return;
 }
@@ -166,18 +193,12 @@ sub finish () {
 ##
 ###############################################################################
 
-sub mce_step_f (@) {
+sub run_file (@) {
+
+   shift if (defined $_[0] && $_[0] eq 'MCE::Step');
 
    my ($_file, $_pos); my $_start_pos = (ref $_[0] eq 'HASH') ? 2 : 1;
 
-   for ($_start_pos .. @_ - 1) {
-      my $_r = ref $_[$_];
-      if ($_r eq "" || $_r eq 'GLOB' || $_r eq 'SCALAR' || $_r =~ /^IO::/) {
-         $_file = $_[$_]; $_pos = $_;
-         last;
-      }
-   }
-
    if (defined $_params) {
       delete $_params->{input_data} if (exists $_params->{input_data});
       delete $_params->{sequence}   if (exists $_params->{sequence});
@@ -186,24 +207,32 @@ sub mce_step_f (@) {
       $_params = {};
    }
 
-   if (defined $_file && ref $_file eq "" && $_file ne "") {
-      _croak("$_tag: '$_file' does not exist") unless (-e $_file);
-      _croak("$_tag: '$_file' is not readable") unless (-r $_file);
-      _croak("$_tag: '$_file' is not a plain file") unless (-f $_file);
+   for my $_i ($_start_pos .. @_ - 1) {
+      my $_r = ref $_[$_i];
+      if ($_r eq '' || $_r eq 'GLOB' || $_r eq 'SCALAR' || $_r =~ /^IO::/) {
+         $_file = $_[$_i]; $_pos = $_i;
+         last;
+      }
+   }
+
+   if (defined $_file && ref $_file eq '' && $_file ne '') {
+      _croak("$_tag: ($_file) does not exist") unless (-e $_file);
+      _croak("$_tag: ($_file) is not readable") unless (-r $_file);
+      _croak("$_tag: ($_file) is not a plain file") unless (-f $_file);
       $_params->{_file} = $_file;
    }
    elsif (ref $_file eq 'GLOB' || ref $_file eq 'SCALAR' || ref($_file) =~ /^IO::/) {
       $_params->{_file} = $_file;
    }
    else {
-      _croak("$_tag: 'file' is not specified or valid");
+      _croak("$_tag: (file) is not specified or valid");
    }
 
    if (defined $_pos) {
       pop @_ for ($_pos .. @_ - 1);
    }
 
-   return mce_step(@_);
+   return run(@_);
 }
 
 ###############################################################################
@@ -212,20 +241,28 @@ sub mce_step_f (@) {
 ##
 ###############################################################################
 
-sub mce_step_s (@) {
+sub run_seq (@) {
+
+   shift if (defined $_[0] && $_[0] eq 'MCE::Step');
 
    my ($_begin, $_end, $_pos); my $_start_pos = (ref $_[0] eq 'HASH') ? 2 : 1;
 
-   delete $_params->{sequence}
-      if (exists $_params->{sequence});
+   if (defined $_params) {
+      delete $_params->{sequence}   if (exists $_params->{sequence});
+      delete $_params->{input_data} if (exists $_params->{input_data});
+      delete $_params->{_file}      if (exists $_params->{_file});
+   }
+   else {
+      $_params = {};
+   }
 
-   for ($_start_pos .. @_ - 1) {
-      my $_ref = ref $_[$_];
+   for my $_i ($_start_pos .. @_ - 1) {
+      my $_ref = ref $_[$_i];
 
-      if ($_ref eq "" || $_ref eq 'HASH' || $_ref eq 'ARRAY') {
-         $_pos = $_;
+      if ($_ref eq '' || $_ref eq 'HASH' || $_ref eq 'ARRAY') {
+         $_pos = $_i;
 
-         if ($_ref eq "") {
+         if ($_ref eq '') {
             $_begin = $_[$_pos]; $_end = $_[$_pos + 1];
             $_params->{sequence} = [
                $_[$_pos], $_[$_pos + 1], $_[$_pos + 2], $_[$_pos + 3]
@@ -244,28 +281,22 @@ sub mce_step_s (@) {
       }
    }
 
-   if (defined $_params) {
-      delete $_params->{input_data} if (exists $_params->{input_data});
-      delete $_params->{_file}      if (exists $_params->{_file});
-   }
-   else {
-      $_params = {};
-   }
-
-   _croak("$_tag: 'sequence' is not specified or valid")
+   _croak("$_tag: (sequence) is not specified or valid")
       unless (exists $_params->{sequence});
 
-   _croak("$_tag: 'begin' is not specified for sequence")
+   _croak("$_tag: (begin) is not specified for sequence")
       unless (defined $_begin);
 
-   _croak("$_tag: 'end' is not specified for sequence")
+   _croak("$_tag: (end) is not specified for sequence")
       unless (defined $_end);
 
+   $_params->{sequence_run} = 1;
+
    if (defined $_pos) {
       pop @_ for ($_pos .. @_ - 1);
    }
 
-   return mce_step(@_);
+   return run(@_);
 }
 
 ###############################################################################
@@ -274,7 +305,9 @@ sub mce_step_s (@) {
 ##
 ###############################################################################
 
-sub mce_step (@) {
+sub run (@) {
+
+   shift if (defined $_[0] && $_[0] eq 'MCE::Step');
 
    if (MCE->wid) {
       @_ = (); _croak(
@@ -284,7 +317,9 @@ sub mce_step (@) {
 
    if (ref $_[0] eq 'HASH') {
       $_params = {} unless defined $_params;
-      $_params->{$_} = $_[0]->{$_} foreach (keys %{ $_[0] });
+      for my $_p (keys %{ $_[0] }) {
+         $_params->{$_p} = $_[0]->{$_p};
+      }
 
       shift;
    }
@@ -344,18 +379,20 @@ sub mce_step (@) {
       delete $_p->{user_tasks} if (exists $_p->{user_tasks});
    }
 
-   $_max_workers = int($_max_workers / @_code + 0.5) + 1
-      if (@_code > 1);
+   if (@_code > 1 && $_max_workers > 1) {
+      $_max_workers = int($_max_workers / @_code + 0.5) + 1;
+   }
 
    my $_chunk_size = MCE::Util::_parse_chunk_size(
       $CHUNK_SIZE, $_max_workers, $_params, $_input_data, scalar @_
    );
 
    if (defined $_params) {
-      $_input_data = $_params->{input_data} if (exists $_params->{input_data});
-
       if (exists $_params->{_file}) {
-         $_input_data = $_params->{_file}; delete $_params->{_file};
+         $_input_data = delete $_params->{_file};
+      }
+      else {
+         $_input_data = $_params->{input_data} if exists $_params->{input_data};
       }
    }
 
@@ -367,27 +404,30 @@ sub mce_step (@) {
       $_MCE->shutdown() if (defined $_MCE);
 
       pop( @_queue )->DESTROY for (@_code .. @_queue);
-      push @_queue, MCE::Queue->new for (@_queue .. @_code - 2);
+
+      push @_queue, MCE::Queue->new(fast => $FAST)
+         for (@_queue .. @_code - 2);
 
       _gen_user_tasks(\@_queue, \@_code, \@_name, \@_wrks, $_chunk_size);
       $_last_task_id = @_code - 1;
 
       my %_options = (
          max_workers => $_max_workers, task_name => $_tag,
-         user_tasks => \@_user_tasks, task_end => \&_task_end
+         user_tasks => \@_user_tasks, task_end => \&_task_end,
       );
 
       if (defined $_params) {
-         my $_p = $_params;
+         local $_; my $_p = $_params;
 
-         foreach (keys %{ $_p }) {
+         for (keys %{ $_p }) {
+            next if ($_ eq 'sequence_run');
             next if ($_ eq 'max_workers' && ref $_p->{max_workers} eq 'ARRAY');
             next if ($_ eq 'task_name' && ref $_p->{task_name} eq 'ARRAY');
             next if ($_ eq 'input_data');
             next if ($_ eq 'chunk_size');
             next if ($_ eq 'task_end');
 
-            _croak("MCE::Step: '$_' is not a valid constructor argument")
+            _croak("MCE::Step: ($_) is not a valid constructor argument")
                unless (exists $MCE::_valid_fields_new{$_});
 
             $_options{$_} = $_p->{$_};
@@ -397,16 +437,16 @@ sub mce_step (@) {
       $_MCE = MCE->new(%_options);
    }
    else {
-      ## Workers may persist after running. MCE::Step allows options
-      ## to be passed using an anonymous hash. Therefore, lets update
-      ## the MCE instance. These options do not require respawning.
-
-      for (qw(
-         RS interval stderr_file stdout_file user_error user_output
-         job_delay submit_delay on_post_exit on_post_run user_args
-         flush_file flush_stderr flush_stdout gather
-      )) {
-         $_MCE->{$_} = $_params->{$_} if (exists $_params->{$_});
+      ## Workers may persist after running. Thus, updating the MCE instance.
+      ## These options do not require respawning.
+      if (defined $_params) {
+         for my $_p (qw(
+            RS interval stderr_file stdout_file user_error user_output
+            job_delay submit_delay on_post_exit on_post_run user_args
+            flush_file flush_stderr flush_stdout gather
+         )) {
+            $_MCE->{$_p} = $_params->{$_p} if (exists $_params->{$_p});
+         }
       }
    }
 
@@ -415,19 +455,40 @@ sub mce_step (@) {
    my @_a; my $_wa = wantarray; $_MCE->{gather} = \@_a if (defined $_wa);
 
    if (defined $_input_data) {
-      @_ = (); $_MCE->process({ chunk_size => $_chunk_size }, $_input_data);
+      @_ = ();
+      $_MCE->process({ chunk_size => $_chunk_size }, $_input_data);
+      delete $_MCE->{input_data};
    }
    elsif (scalar @_) {
       $_MCE->process({ chunk_size => $_chunk_size }, \@_);
+      delete $_MCE->{input_data};
    }
    else {
-      $_MCE->run({ chunk_size => $_chunk_size }, 0);
+      if (defined $_params && exists $_params->{sequence}) {
+         $_MCE->run({
+            chunk_size => $_chunk_size, sequence => $_params->{sequence}
+         }, 0);
+         if (exists $_params->{sequence_run}) {
+            delete $_params->{sequence_run};
+            delete $_params->{sequence};
+         }
+         delete $_MCE->{sequence};
+      }
+      else {
+         $_MCE->run({ chunk_size => $_chunk_size }, 0);
+      }
    }
 
    delete $_MCE->{gather} if (defined $_wa);
 
    MCE::_restore_state;
 
+   if (exists $_MCE->{_rla_return}) {
+      $MCE::MCE->{_rla_return} = delete $_MCE->{_rla_return};
+   }
+
+   finish() if ($^S);   ## shutdown if in eval state
+
    return ((defined $_wa) ? @_a : ());
 }
 
@@ -452,12 +513,22 @@ sub _gen_user_func {
    return sub {
       my ($_mce) = @_;
 
-      while (defined (my $_chunk = $_q_in->dequeue())) {
-         $_chunk  = $_mce->thaw($_chunk);
-         local $_ = $_chunk->[0];
-         $_c_ref->($_mce, @$_chunk);
+      $_mce->{_next_jmp} = sub { goto _MCE_STEP__NEXT; };
+      $_mce->{_last_jmp} = sub { goto _MCE_STEP__LAST; };
+
+      _MCE_STEP__NEXT:
+
+      while (defined (local $_ = $_q_in->dequeue())) {
+         if (chop $_) {
+            my $_args = $_mce->thaw($_);  $_ = $_args->[0];
+            $_c_ref->($_mce, @{ $_args });
+         } else {
+            $_c_ref->($_mce, $_);
+         }
       }
 
+      _MCE_STEP__LAST:
+
       return;
    };
 }
@@ -489,12 +560,15 @@ sub _gen_user_tasks {
 
 sub _validate_number {
 
-   my $_n = $_[0]; my $_key = $_[1];
+   my ($_n, $_key) = @_;
+
+   _croak("$_tag: ($_key) is not valid") if (!defined $_n);
 
    $_n =~ s/K\z//i; $_n =~ s/M\z//i;
 
-   _croak("$_tag: '$_key' is not valid")
-      if (!looks_like_number($_n) || int($_n) != $_n || $_n < 1);
+   if (!looks_like_number($_n) || int($_n) != $_n || $_n < 1) {
+      _croak("$_tag: ($_key) is not valid");
+   }
 
    return;
 }
@@ -515,15 +589,15 @@ MCE::Step - Parallel step model for building creative steps
 
 =head1 VERSION
 
-This document describes MCE::Step version 1.517
+This document describes MCE::Step version 1.608
 
 =head1 DESCRIPTION
 
-MCE::Step is similar to L<MCE::Flow> for writing custom apps to maximize on
-all available cores. The main difference comes from the transparent inclusion
-of queues between sub-tasks.
+MCE::Step is similar to L<MCE::Flow|MCE::Flow> for writing custom apps.
+The main difference comes from the transparent use of queues between
+sub-tasks.
 
-It's trivial to parallelize with mce_stream as shown below.
+It is trivial to parallelize with mce_stream shown below.
 
    ## Native map function
    my @a = map { $_ * 4 } map { $_ * 3 } map { $_ * 2 } 1..10000;
@@ -537,27 +611,23 @@ It's trivial to parallelize with mce_stream as shown below.
         sub { $_ * 4 }, sub { $_ * 3 }, sub { $_ * 2 }, 1..10000;
 
 However, let's have MCE::Step compute the same in parallel. Unlike the example
-shown in L<MCE::Flow>, the MCE::Queue objects are created and managed for you
-automatically. The use of MCE::Queue is totally transparent.
+in L<MCE::Flow|MCE::Flow>, the use of MCE::Queue is totally transparent.
 
    use MCE::Step;
 
-This calls for preserving output order. This time, will call on a function to
-emit a closure, the iterator itself for the gather option. The closure saves
-one from having to re-initialize $order_id prior to each run.
+This calls for preserving output order.
 
-   sub output_iterator {
+   sub preserve_order {
       my %tmp; my $order_id = 1; my $gather_ref = $_[0];
-
-      @{ $gather_ref } = ();     ## Optional: clear the array
+      @{ $gather_ref } = ();  ## clear the array (optional)
 
       return sub {
-         $tmp{ $_[1] } = $_[0];
+         my ($data_ref, $chunk_id) = @_;
+         $tmp{$chunk_id} = $data_ref;
 
          while (1) {
             last unless exists $tmp{$order_id};
-            push @{ $gather_ref }, @{ $tmp{$order_id} };
-            delete $tmp{$order_id++};
+            push @{ $gather_ref }, @{ delete $tmp{$order_id++} };
          }
 
          return;
@@ -565,7 +635,7 @@ one from having to re-initialize $order_id prior to each run.
    }
 
 Next are the 3 sub-tasks. Compare these 3 sub-tasks with the same as described
-in L<MCE::Flow>. The call to MCE->step is all that's needed for passing data
+in L<MCE::Flow|MCE::Flow>. The call to MCE->step simplifies the passing of data
 into the next sub-task.
 
    sub task_a {
@@ -587,75 +657,76 @@ into the next sub-task.
    }
 
 In summary, MCE::Step builds out a MCE instance behind the scene and starts
-running. Both task_name (shown below) and max_workers can take an anonymous
-array for specifying the values individually for each sub-task.
+running. Both task_name and max_workers (not shown) can take an anonymous
+array for specifying the values uniquely for each sub-task.
 
    my @a;
 
    mce_step {
-      gather => output_iterator(\@a), task_name => [ 'a', 'b', 'c' ]
+      task_name => [ 'a', 'b', 'c' ],
+      gather => preserve_order(\@a)
 
    }, \&task_a, \&task_b, \&task_c, 1..10000;
 
    print "@a\n";
 
-=head1 GATHER AND STEP DEMO
+=head1 STEP DEMO
 
-One may call gather or step any number of times although calling step is not
-allowed from the very last sub-block. There's a lot going on below. For one,
-STDOUT output is serialized back to the main process. We see that chunk_size
-is specified when loading the module and set to 1 for the demonstration.
+In the demonstration below, one may call ->gather or ->step any number of times
+although ->step is not allowed in the last sub-block. Data is gathered to @arr
+which may likely be out-of-order. Gathering data is optional. All sub-blocks
+receive $mce as the first argument.
 
-Data is gathered to @arr which may likely be out-of-order. Gathering data is
-optional and not a requirement to use MCE::Step. Note that all sub-tasks
-receive the $mce instance as the very first argument.
+First, defining 3 sub-tasks.
 
-The tasks below run in parallel, and each with multiple workers as well.
-One may copy this code snippet and add MCE->task_wid or MCE->wid to the
-output. Remember that max_workers can take an anonymous array for specifying
-max_workers individually per each task block, similarly to task_name in the
-previous section.
+   use MCE::Step;
 
-   use MCE::Step chunk_size => 1;
-
-   my @arr = mce_step
-
-   sub {
+   sub task_a {
       my ($mce, $chunk_ref, $chunk_id) = @_;
 
       if ($_ % 2 == 0) {
          MCE->gather($_);
-       # MCE->gather($_ * 4);     ## Ok to gather multiple times
+       # MCE->gather($_ * 4);        ## Ok to gather multiple times
       }
       else {
          MCE->print("a step: $_, $_ * $_\n");
          MCE->step($_, $_ * $_);
-       # MCE->step($_, $_ * 4 );  ## Ok to step multiple times
+       # MCE->step($_, $_ * 4 );     ## Ok to step multiple times
       }
-   },
+   }
 
-   sub {
+   sub task_b {
       my ($mce, $arg1, $arg2) = @_;
 
       MCE->print("b args: $arg1, $arg2\n");
 
-      if ($_ % 3 == 0) {          ## $_ is the same as $arg1
+      if ($_ % 3 == 0) {             ## $_ is the same as $arg1
          MCE->gather($_);
       }
       else {
          MCE->print("b step: $_ * $_\n");
          MCE->step($_ * $_);
       }
-   },
+   }
 
-   sub {
+   sub task_c {
       my ($mce, $arg1) = @_;
 
       MCE->print("c: $_\n");
       MCE->gather($_);
-   },
+   }
 
-   1..10;
+Next, pass MCE options, using chunk_size 1, and run all 3 tasks in parallel.
+Notice how max_workers can take an anonymous array, similarly to task_name.
+
+   my @arr = mce_step {
+      task_name   => [ 'a', 'b', 'c' ],
+      max_workers => [  2,   2,   2  ],
+      chunk_size  => 1
+
+   }, \&task_a, \&task_b, \&task_c, 1..10;
+
+Finally, sort the array and display its contents.
 
    @arr = sort { $a <=> $b } @arr;
 
@@ -684,11 +755,12 @@ previous section.
 
 =head1 SYNOPSIS when CHUNK_SIZE EQUALS 1
 
-Although L<MCE::Loop> may be preferred for running using a single code block,
-the text below also applies to this module, particularly for the first block.
+Although L<MCE::Loop|MCE::Loop> may be preferred for running using a single
+code block, the text below also applies to this module, particularly for the
+first block.
 
 All models in MCE default to 'auto' for chunk_size. The arguments for the block
-are the same as writing a user_func block for the core API.
+are the same as writing a user_func block using the Core API.
 
 Beginning with MCE 1.5, the next input item is placed into the input scalar
 variable $_ when chunk_size equals 1. Otherwise, $_ points to $chunk_ref
@@ -734,7 +806,7 @@ inside the first block. Hence, the block is called once per each item.
 
 =head1 SYNOPSIS when CHUNK_SIZE is GREATER THAN 1
 
-Follow this synopsis when chunk_size equals 'auto' or is greater than 1.
+Follow this synopsis when chunk_size equals 'auto' or greater than 1.
 This means having to loop through the chunk from inside the first block.
 
    use MCE::Step;
@@ -749,7 +821,7 @@ This means having to loop through the chunk from inside the first block.
 
    mce_step sub { do_work($_) for (@{ $_ }) }, 1..10000;
 
-   ## Same as above, resembles code using the core API.
+   ## Same as above, resembles code using the Core API.
 
    mce_step sub {
       my ($mce, $chunk_ref, $chunk_id) = @_;
@@ -766,21 +838,24 @@ choosing 1 for chunk_size is fine.
 
 =head1 OVERRIDING DEFAULTS
 
-The following list 5 options which may be overridden when loading the module.
+The following list 6 options which may be overridden when loading the module.
 
-   use Sereal qw(encode_sereal decode_sereal);
+   use Sereal qw( encode_sereal decode_sereal );
+   use CBOR::XS qw( encode_cbor decode_cbor );
+   use JSON::XS qw( encode_json decode_json );
 
    use MCE::Step
-         max_workers => 8,                    ## Default 'auto'
-         chunk_size  => 500,                  ## Default 'auto'
-         tmp_dir     => "/path/to/app/tmp",   ## $MCE::Signal::tmp_dir
-         freeze      => \&encode_sereal,      ## \&Storable::freeze
-         thaw        => \&decode_sereal       ## \&Storable::thaw
+         max_workers => 8,               ## Default 'auto'
+         chunk_size => 500,              ## Default 'auto'
+         fast => 1,                      ## Default 0 (fast queue?)
+         tmp_dir => "/path/to/app/tmp",  ## $MCE::Signal::tmp_dir
+         freeze => \&encode_sereal,      ## \&Storable::freeze
+         thaw => \&decode_sereal         ## \&Storable::thaw
    ;
 
 There is a simpler way to enable Sereal with MCE 1.5. The following will
-attempt to use Sereal if available, otherwise will default back to using
-Storable for serialization.
+attempt to use Sereal if available, otherwise defaults to Storable for
+serialization.
 
    use MCE::Step Sereal => 1;
 
@@ -788,17 +863,19 @@ Storable for serialization.
       chunk_size => 1
    };
 
-   ## Serialization is through Sereal if available.
+   ## Serialization is by the Sereal module if available.
    my %answer = mce_step sub { MCE->gather( $_, sqrt $_ ) }, 1..10000;
 
 =head1 CUSTOMIZING MCE
 
 =over 3
 
-=item init
+=item MCE::Step->init ( options )
+
+=item MCE::Step::init { options }
 
 The init function accepts a hash of MCE options. Unlike with MCE::Stream,
-both the gather and bounds_only options may be specified when calling init
+both gather and bounds_only options may be specified when calling init
 (not shown below).
 
    use MCE::Step;
@@ -843,26 +920,28 @@ both the gather and bounds_only options may be specified when calling init
 
 Like with MCE::Step::init above, MCE options may be specified using an
 anonymous hash for the first argument. Notice how both max_workers and
-task_name can take an anonymous array for setting values individually
+task_name can take an anonymous array for setting values uniquely
 for each code block.
 
 Unlike MCE::Stream which processes from right-to-left, MCE::Step begins
 with the first code block, thus processing from left-to-right.
 
-The following script takes 9 seconds to complete. Removing both calls to
-MCE->step will cause the script to complete in just 1 second. The reason is
-due to the 2nd and subsequent sub-tasks awaiting data from their queues.
-Workers terminate internally when receiving an undef from the queue. The 9
-seconds is from having only 2 workers assigned for the last sub-task and
-waiting 1 or 2 seconds initially before calling MCE->step.
+The following takes 9 seconds to complete. The 9 seconds is from having
+only 2 workers assigned for the last sub-task and waiting 1 or 2 seconds
+initially before calling MCE->step.
+
+Removing both calls to MCE->step will cause the script to complete in just
+1 second. The reason is due to the 2nd and subsequent sub-tasks awaiting
+data from an internal queue. Workers terminate upon receiving an undef.
 
    use MCE::Step;
 
    my @a = mce_step {
-      max_workers => [ 3, 4, 2, ], task_name => [ 'a', 'b', 'c' ],
+      task_name   => [ 'a', 'b', 'c' ],
+      max_workers => [  3,   4,   2, ],
 
       user_end => sub {
-         my ($task_id, $task_name) = (MCE->task_id, MCE->task_name);
+         my ($mce, $task_id, $task_name) = @_;
          MCE->print("$task_id - $task_name completed\n");
       },
 
@@ -893,17 +972,38 @@ waiting 1 or 2 seconds initially before calling MCE->step.
 =head1 API DOCUMENTATION
 
 Although input data is optional for MCE::Step, the following assumes chunk_size
-equals 1 in order to demonstrate all the possibilities of passing input data.
+equals 1 in order to demonstrate all the possibilities of passing input data
+into the code block.
 
 =over 3
 
+=item MCE::Step->run ( { input_data => iterator }, sub { code } )
+
+=item mce_step { input_data => iterator }, sub { code }
+
+An iterator reference can by specified for input_data. The only other way
+is to specify input_data via MCE::Step::init. This prevents MCE::Step from
+configuring the iterator reference as another user task which will not work.
+
+Iterators are described under "SYNTAX for INPUT_DATA" at L<MCE::Core|MCE::Core>.
+
+   MCE::Step::init {
+      input_data => iterator
+   };
+
+   mce_step sub { $_ };
+
+=item MCE::Step->run ( sub { code }, list )
+
 =item mce_step sub { code }, list
 
-Input data can be defined using a list or passing a reference to an array.
+Input data can be defined using a list.
 
    mce_step sub { $_ }, 1..1000;
    mce_step sub { $_ }, [ 1..1000 ];
 
+=item MCE::Step->run_file ( sub { code }, file )
+
 =item mce_step_f sub { code }, file
 
 The fastest of these is the /path/to/file. Workers communicate the next offset
@@ -913,7 +1013,9 @@ position among themselves without any interaction from the manager process.
    mce_step_f sub { $_ }, $file_handle;
    mce_step_f sub { $_ }, \$scalar;
 
-=item mce_step_s sub { code }, sequence
+=item MCE::Step->run_seq ( sub { code }, $beg, $end [, $step, $fmt ] )
+
+=item mce_step_s sub { code }, $beg, $end [, $step, $fmt ]
 
 Sequence can be defined as a list, an array reference, or a hash reference.
 The functions require both begin and end values to run. Step and format are
@@ -928,58 +1030,41 @@ optional. The format is passed to sprintf (% may be omitted below).
       begin => $beg, end => $end, step => $step, format => $fmt
    };
 
-=item mce_step { input_data => iterator }, sub { code }
-
-An iterator reference can by specified for input data. Notice the anonymous
-hash as the first argument to mce_step. The only other way is to specify
-input_data via MCE::Step::init. This prevents MCE::Step from configuring
-the iterator reference as another user task which will not work.
-
-Iterators are described under "SYNTAX for INPUT_DATA" at L<MCE::Core>.
-
-   MCE::Step::init {
-      input_data => iterator
-   };
-
-   mce_step sub { $_ };
-
 =back
 
-The sequence engine can compute the 'begin' and 'end' items only, for the chunk,
-leaving out the items in between with the bounds_only option (boundaries only).
-This option applies to sequence and has no effect when chunk_size equals 1.
+The sequence engine can compute 'begin' and 'end' items only, for the chunk,
+and not the items in between (hence boundaries only). This option applies
+to sequence only and has no effect when chunk_size equals 1.
 
-The time to run for MCE below is 0.013s. This becomes 0.834s without the
-bounds_only option due to computing all items in between as well, thus
-creating a very large array. Basically, specify bounds_only => 1 when
-boundaries is all you need for looping inside the block; e.g Monte Carlo
-simulations. Time was measured using 1 worker to emphasize the difference.
+The time to run is 0.006s below. This becomes 0.827s without the bounds_only
+option due to computing all items in between, thus creating a very large
+array. Basically, specify bounds_only => 1 when boundaries is all you need
+for looping inside the block; e.g. Monte Carlo simulations.
+
+Time was measured using 1 worker to emphasize the difference.
 
    use MCE::Step;
 
    MCE::Step::init {
-      max_workers => 1,
-    # chunk_size  => 'auto',     ## btw, 'auto' will never drop below 2
-      chunk_size  => 1_250_000,
+      max_workers => 1, chunk_size => 1_250_000,
       bounds_only => 1
    };
 
    ## For sequence, the input scalar $_ points to $chunk_ref
-   ## when chunk_size > 1, otherwise equals $chunk_ref->[0].
+   ## when chunk_size > 1, otherwise $chunk_ref->[0].
    ##
    ## mce_step_s sub {
    ##    my $begin = $_->[0]; my $end = $_->[-1];
    ##
    ##    for ($begin .. $end) {
-   ##       ... have fun with MCE ...
+   ##       ...
    ##    }
    ##
    ## }, 1, 10_000_000;
 
    mce_step_s sub {
       my ($mce, $chunk_ref, $chunk_id) = @_;
-
-      ## $chunk_ref contains just 2 items, not 1_250_000
+      ## $chunk_ref contains 2 items, not 1_250_000
 
       my $begin = $chunk_ref->[ 0];
       my $end   = $chunk_ref->[-1];   ## or $chunk_ref->[1]
@@ -1010,8 +1095,7 @@ the gather method is used to have results sent back to the manager process.
    my @a = mce_step sub { MCE->gather($_ * 2) }, 1..100;
    print "@a\n\n";
 
-   ## However, one can store to a hash by gathering 2 items per
-   ## each gather call (key, value).
+   ## Outputs to a hash instead (key, value).
    my %h1 = mce_step sub { MCE->gather($_, $_ * 2) }, 1..100;
    print "@h1{1..100}\n\n";
 
@@ -1080,7 +1164,6 @@ The following uses an anonymous array containing 3 elements when gathering
 data. Serialization is automatic behind the scene.
 
    my %h3 = mce_step sub {
-
       ...
 
       MCE->gather($host, [$output, $error, $status]);
@@ -1093,45 +1176,41 @@ data. Serialization is automatic behind the scene.
       print "Exit status: ", $h3{$host}->[2], "\n\n";
    }
 
-Perhaps you want more control with gather such as appending to an array while
-retaining output order. Although MCE::Map comes to mind, some folks want "full"
-control. And here we go... but this time around in chunking style... :)
+Although MCE::Map comes to mind, one may want additional control when
+gathering data such as retaining output order.
 
-The two options passed to MCE::Step are optional as they default to 'auto'. The
-beauty of chunking data is that IPC occurs once per chunk versus once per item.
-Although IPC is quite fast, chunking becomes beneficial the larger the data
-becomes. Hence, the reason for the demonstration below.
-
-   use MCE::Step chunk_size => 'auto', max_workers => 'auto';
-
-   my (%_tmp, $_gather_ref, $_order_id);
+   use MCE::Step;
 
    sub preserve_order {
-      $_tmp{ (shift) } = \@_;
+      my %tmp; my $order_id = 1; my $gather_ref = $_[0];
 
-      while (1) {
-         last unless exists $_tmp{$_order_id};
-         push @{ $_gather_ref }, @{ $_tmp{$_order_id} };
-         delete $_tmp{$_order_id++};
-      }
+      return sub {
+         $tmp{ (shift) } = \@_;
 
-      return;
+         while (1) {
+            last unless exists $tmp{$order_id};
+            push @{ $gather_ref }, @{ delete $tmp{$order_id++} };
+         }
+
+         return;
+      };
    }
 
-   ## Workers persist after running. Therefore, not recommended to
-   ## use a closure for gather unless calling MCE::Step::init each
-   ## time inside the loop. Use this demonstration when wanting
-   ## MCE::Step to maintain output order.
+   ## Workers persist for the most part after running. Though, not always
+   ## the case and depends on Perl. Pass a reference to a subroutine if
+   ## workers must persist; e.g. mce_step { ... }, \&foo, 1..100000.
 
-   MCE::Step::init { gather => \&preserve_order };
+   MCE::Step::init {
+      chunk_size => 'auto', max_workers => 'auto'
+   };
 
    for (1..2) {
       my @m2;
 
-      ## Remember to set $_order_id back to 1 prior to running.
-      $_gather_ref = \@m2; $_order_id = 1;
-
-      mce_step sub {
+      mce_step {
+         gather => preserve_order(\@m2)
+      },
+      sub {
          my @a; my ($mce, $chunk_ref, $chunk_id) = @_;
 
          ## Compute the entire chunk data at once.
@@ -1146,20 +1225,23 @@ becomes. Hence, the reason for the demonstration below.
       print scalar @m2, "\n";
    }
 
-All 6 models support 'auto' for chunk_size whereas the core API doesn't. Think
-of the models as the basis for providing JIT for MCE. They create the instance
-and tune max_workers plus chunk_size automatically irregardless of the
-hardware being run on.
+   MCE::Step::finish;
 
-The following does the same thing using the core API.
+All 6 models support 'auto' for chunk_size unlike the Core API. Think of the
+models as the basis for providing JIT for MCE. They create the instance, tune
+max_workers, and tune chunk_size automatically regardless of the hardware.
+
+The following does the same thing using the Core API. Workers persist after
+running.
 
    use MCE;
 
-   ...
+   sub preserve_order {
+      ...
+   }
 
    my $mce = MCE->new(
       max_workers => 'auto', chunk_size => 8000,
-      gather => \&preserve_order,
 
       user_func => sub {
          my @a; my ($mce, $chunk_ref, $chunk_id) = @_;
@@ -1173,19 +1255,27 @@ The following does the same thing using the core API.
       }
    );
 
-   $mce->process([1..100000]);
+   for (1..2) {
+      my @m2;
 
-   ...
+      $mce->process({ gather => preserve_order(\@m2) }, [1..100000]);
+
+      print scalar @m2, "\n";
+   }
+
+   $mce->shutdown;
 
 =head1 MANUAL SHUTDOWN
 
 =over 3
 
-=item finish
+=item MCE::Step->finish
 
-MCE workers remain persistent as much as possible after running. Shutdown
-occurs when the script exits. One can manually shutdown MCE by simply calling
-finish after running. This resets the MCE instance.
+=item MCE::Step::finish
+
+Workers remain persistent as much as possible after running. Shutdown occurs
+automatically when the script terminates. Call finish when workers are no
+longer needed.
 
    use MCE::Step;
 
@@ -1201,19 +1291,11 @@ finish after running. This resets the MCE instance.
 
 =head1 INDEX
 
-L<MCE>
+L<MCE|MCE>
 
 =head1 AUTHOR
 
 Mario E. Roy, S<E<lt>marioeroy AT gmail DOT comE<gt>>
 
-=head1 LICENSE
-
-This program is free software; you can redistribute it and/or modify it
-under the terms of either: the GNU General Public License as published
-by the Free Software Foundation; or the Artistic License.
-
-See L<http://dev.perl.org/licenses/> for more information.
-
 =cut
 
