@@ -9,14 +9,18 @@ package MCE::Stream;
 use strict;
 use warnings;
 
+## no critic (BuiltinFunctions::ProhibitStringyEval)
+## no critic (Subroutines::ProhibitSubroutinePrototypes)
+## no critic (TestingAndDebugging::ProhibitNoStrict)
+
 use Scalar::Util qw( looks_like_number );
 
 use MCE;
-use MCE::Util;
-
 use MCE::Queue;
 
-our $VERSION = '1.517'; $VERSION = eval $VERSION;
+our $VERSION  = '1.608';
+
+our @CARP_NOT = qw( MCE );
 
 ###############################################################################
 ## ----------------------------------------------------------------------------
@@ -24,9 +28,10 @@ our $VERSION = '1.517'; $VERSION = eval $VERSION;
 ##
 ###############################################################################
 
-our $DEFAULT_MODE = 'map';
-our $MAX_WORKERS  = 'auto';
-our $CHUNK_SIZE   = 'auto';
+my $DEFAULT_MODE = 'map';
+my $MAX_WORKERS  = 'auto';
+my $CHUNK_SIZE   = 'auto';
+my $FAST         = 0;
 
 my ($_params, @_prev_c, @_prev_m, @_prev_n, @_prev_w, @_user_tasks, @_queue);
 my ($_MCE, $_loaded); my $_tag = 'MCE::Stream';
@@ -36,30 +41,47 @@ sub import {
    my $_class = shift; return if ($_loaded++);
 
    ## Process module arguments.
-   while (my $_arg = shift) {
+   while (my $_argument = shift) {
+      my $_arg = lc $_argument;
 
-      $DEFAULT_MODE = shift and next if ( $_arg =~ /^default_mode$/i );
-      $MAX_WORKERS  = shift and next if ( $_arg =~ /^max_workers$/i );
-      $CHUNK_SIZE   = shift and next if ( $_arg =~ /^chunk_size$/i );
-      $MCE::TMP_DIR = shift and next if ( $_arg =~ /^tmp_dir$/i );
-      $MCE::FREEZE  = shift and next if ( $_arg =~ /^freeze$/i );
-      $MCE::THAW    = shift and next if ( $_arg =~ /^thaw$/i );
+      $DEFAULT_MODE = shift and next if ( $_arg eq 'default_mode' );
+      $MAX_WORKERS  = shift and next if ( $_arg eq 'max_workers' );
+      $CHUNK_SIZE   = shift and next if ( $_arg eq 'chunk_size' );
 
-      if ( $_arg =~ /^sereal$/i ) {
+      $MCE::FREEZE = $MCE::MCE->{freeze} = shift and next
+         if ( $_arg eq 'freeze' );
+      $MCE::THAW = $MCE::MCE->{thaw} = shift and next
+         if ( $_arg eq 'thaw' );
+
+      if ( $_arg eq 'sereal' ) {
          if (shift eq '1') {
             local $@; eval 'use Sereal qw(encode_sereal decode_sereal)';
             unless ($@) {
-               $MCE::FREEZE = \&encode_sereal;
-               $MCE::THAW   = \&decode_sereal;
+               $MCE::FREEZE = $MCE::MCE->{freeze} = \&encode_sereal;
+               $MCE::THAW = $MCE::MCE->{thaw} = \&decode_sereal;
             }
          }
          next;
       }
 
-      _croak("$_tag::import: '$_arg' is not a valid module argument");
+      if ( $_arg eq 'tmp_dir' ) {
+         $MCE::TMP_DIR = $MCE::MCE->{tmp_dir} = shift;
+         my $_e1 = 'is not a directory or does not exist';
+         my $_e2 = 'is not writeable';
+         _croak($_tag."::import: ($MCE::TMP_DIR) $_e1") unless -d $MCE::TMP_DIR;
+         _croak($_tag."::import: ($MCE::TMP_DIR) $_e2") unless -w $MCE::TMP_DIR;
+         next;
+      }
+
+      if ( $_arg eq 'fast' ) {
+         $FAST = 1 if (shift eq '1');
+         next;
+      }
+
+      _croak($_tag."::import: ($_argument) is not a valid module argument");
    }
 
-   _croak("$_tag: 'DEFAULT_MODE' is not valid")
+   _croak("$_tag: (DEFAULT_MODE) is not valid")
       if ($DEFAULT_MODE ne 'grep' && $DEFAULT_MODE ne 'map');
 
    $MAX_WORKERS = MCE::Util::_parse_max_workers($MAX_WORKERS);
@@ -70,11 +92,11 @@ sub import {
 
    ## Import functions.
    no strict 'refs'; no warnings 'redefine';
-   my $_package = caller();
+   my $_pkg = caller;
 
-   *{ $_package . '::mce_stream_f' } = \&mce_stream_f;
-   *{ $_package . '::mce_stream_s' } = \&mce_stream_s;
-   *{ $_package . '::mce_stream'   } = \&mce_stream;
+   *{ $_pkg.'::mce_stream_f' } = \&run_file;
+   *{ $_pkg.'::mce_stream_s' } = \&run_seq;
+   *{ $_pkg.'::mce_stream'   } = \&run;
 
    return;
 }
@@ -82,7 +104,7 @@ sub import {
 END {
    return if (defined $_MCE && $_MCE->wid);
 
-   MCE::Stream::finish();
+   finish();
 }
 
 ###############################################################################
@@ -101,8 +123,7 @@ sub _preserve_order {
    if (defined $_gather_ref) {
       while (1) {
          last unless exists $_tmp{$_order_id};
-         push @{ $_gather_ref }, @{ $_tmp{$_order_id} };
-         delete $_tmp{$_order_id++};
+         push @{ $_gather_ref }, @{ delete $_tmp{$_order_id++} };
       }
    }
    else {
@@ -117,10 +138,10 @@ sub _task_end {
    my ($_mce, $_task_id, $_task_name) = @_;
 
    if (defined $_mce->{user_tasks}->[$_task_id + 1]) {
-      my $N_workers = $_mce->{user_tasks}->[$_task_id + 1]->{max_workers};
+      my $n_workers = $_mce->{user_tasks}->[$_task_id + 1]->{max_workers};
       my $_queue_id = @_queue - $_task_id - 1;
 
-      $_queue[$_queue_id]->enqueue((undef) x $N_workers);
+      $_queue[$_queue_id]->enqueue((undef) x $n_workers);
    }
 
    $_params->{task_end}->($_mce, $_task_id, $_task_name)
@@ -137,30 +158,31 @@ sub _task_end {
 
 sub init (@) {
 
+   shift if (defined $_[0] && $_[0] eq 'MCE::Stream');
+
    if (MCE->wid) {
       @_ = (); _croak(
          "$_tag: function cannot be called by the worker process"
       );
    }
 
-   _croak("$_tag: 'argument' is not a HASH reference")
-      unless (ref $_[0] eq 'HASH');
+   finish(); $_params = (ref $_[0] eq 'HASH') ? shift : { @_ };
 
-   MCE::Stream::finish(); $_params = shift;
+   @_ = ();
 
    return;
 }
 
 sub finish () {
 
-   if (defined $_MCE) {
+   if (defined $_MCE && $_MCE->{_spawned}) {
       MCE::_save_state; $_MCE->shutdown(); MCE::_restore_state;
    }
 
    $_gather_ref = $_order_id = undef; undef %_tmp; @_user_tasks = ();
    @_prev_w = (); @_prev_n = (); @_prev_m = (); @_prev_c = ();
 
-   $_->DESTROY() foreach (@_queue); @_queue = ();
+   $_->DESTROY() for (@_queue); @_queue = ();
 
    return;
 }
@@ -171,18 +193,12 @@ sub finish () {
 ##
 ###############################################################################
 
-sub mce_stream_f (@) {
+sub run_file (@) {
+
+   shift if (defined $_[0] && $_[0] eq 'MCE::Stream');
 
    my ($_file, $_pos); my $_start_pos = (ref $_[0] eq 'HASH') ? 2 : 1;
 
-   for ($_start_pos .. @_ - 1) {
-      my $_r = ref $_[$_];
-      if ($_r eq "" || $_r eq 'GLOB' || $_r eq 'SCALAR' || $_r =~ /^IO::/) {
-         $_file = $_[$_]; $_pos = $_;
-         last;
-      }
-   }
-
    if (defined $_params) {
       delete $_params->{input_data} if (exists $_params->{input_data});
       delete $_params->{sequence}   if (exists $_params->{sequence});
@@ -191,24 +207,32 @@ sub mce_stream_f (@) {
       $_params = {};
    }
 
-   if (defined $_file && ref $_file eq "" && $_file ne "") {
-      _croak("$_tag: '$_file' does not exist") unless (-e $_file);
-      _croak("$_tag: '$_file' is not readable") unless (-r $_file);
-      _croak("$_tag: '$_file' is not a plain file") unless (-f $_file);
+   for my $_i ($_start_pos .. @_ - 1) {
+      my $_r = ref $_[$_i];
+      if ($_r eq '' || $_r eq 'GLOB' || $_r eq 'SCALAR' || $_r =~ /^IO::/) {
+         $_file = $_[$_i]; $_pos = $_i;
+         last;
+      }
+   }
+
+   if (defined $_file && ref $_file eq '' && $_file ne '') {
+      _croak("$_tag: ($_file) does not exist") unless (-e $_file);
+      _croak("$_tag: ($_file) is not readable") unless (-r $_file);
+      _croak("$_tag: ($_file) is not a plain file") unless (-f $_file);
       $_params->{_file} = $_file;
    }
    elsif (ref $_file eq 'GLOB' || ref $_file eq 'SCALAR' || ref($_file) =~ /^IO::/) {
       $_params->{_file} = $_file;
    }
    else {
-      _croak("$_tag: 'file' is not specified or valid");
+      _croak("$_tag: (file) is not specified or valid");
    }
 
    if (defined $_pos) {
       pop @_ for ($_pos .. @_ - 1);
    }
 
-   return mce_stream(@_);
+   return run(@_);
 }
 
 ###############################################################################
@@ -217,20 +241,28 @@ sub mce_stream_f (@) {
 ##
 ###############################################################################
 
-sub mce_stream_s (@) {
+sub run_seq (@) {
+
+   shift if (defined $_[0] && $_[0] eq 'MCE::Stream');
 
    my ($_begin, $_end, $_pos); my $_start_pos = (ref $_[0] eq 'HASH') ? 2 : 1;
 
-   delete $_params->{sequence}
-      if (exists $_params->{sequence});
+   if (defined $_params) {
+      delete $_params->{sequence}   if (exists $_params->{sequence});
+      delete $_params->{input_data} if (exists $_params->{input_data});
+      delete $_params->{_file}      if (exists $_params->{_file});
+   }
+   else {
+      $_params = {};
+   }
 
-   for ($_start_pos .. @_ - 1) {
-      my $_ref = ref $_[$_];
+   for my $_i ($_start_pos .. @_ - 1) {
+      my $_ref = ref $_[$_i];
 
-      if ($_ref eq "" || $_ref eq 'HASH' || $_ref eq 'ARRAY') {
-         $_pos = $_;
+      if ($_ref eq '' || $_ref eq 'HASH' || $_ref eq 'ARRAY') {
+         $_pos = $_i;
 
-         if ($_ref eq "") {
+         if ($_ref eq '') {
             $_begin = $_[$_pos]; $_end = $_[$_pos + 1];
             $_params->{sequence} = [
                $_[$_pos], $_[$_pos + 1], $_[$_pos + 2], $_[$_pos + 3]
@@ -249,28 +281,22 @@ sub mce_stream_s (@) {
       }
    }
 
-   if (defined $_params) {
-      delete $_params->{input_data} if (exists $_params->{input_data});
-      delete $_params->{_file}      if (exists $_params->{_file});
-   }
-   else {
-      $_params = {};
-   }
-
-   _croak("$_tag: 'sequence' is not specified or valid")
+   _croak("$_tag: (sequence) is not specified or valid")
       unless (exists $_params->{sequence});
 
-   _croak("$_tag: 'begin' is not specified for sequence")
+   _croak("$_tag: (begin) is not specified for sequence")
       unless (defined $_begin);
 
-   _croak("$_tag: 'end' is not specified for sequence")
+   _croak("$_tag: (end) is not specified for sequence")
       unless (defined $_end);
 
+   $_params->{sequence_run} = 1;
+
    if (defined $_pos) {
       pop @_ for ($_pos .. @_ - 1);
    }
 
-   return mce_stream(@_);
+   return run(@_);
 }
 
 ###############################################################################
@@ -279,7 +305,9 @@ sub mce_stream_s (@) {
 ##
 ###############################################################################
 
-sub mce_stream (@) {
+sub run (@) {
+
+   shift if (defined $_[0] && $_[0] eq 'MCE::Stream');
 
    if (MCE->wid) {
       @_ = (); _croak(
@@ -289,17 +317,19 @@ sub mce_stream (@) {
 
    if (ref $_[0] eq 'HASH' && !exists $_[0]->{code}) {
       $_params = {} unless defined $_params;
-      $_params->{$_} = $_[0]->{$_} foreach (keys %{ $_[0] });
+      for my $_p (keys %{ $_[0] }) {
+         $_params->{$_p} = $_[0]->{$_p};
+      }
 
       shift;
    }
 
-   my $_aref = shift if (ref $_[0] eq 'ARRAY');
+   my $_aref; $_aref = shift if (ref $_[0] eq 'ARRAY');
 
    $_order_id = 1; undef %_tmp;
 
    if (defined $_aref) {
-      $_gather_ref = $_aref; @$_aref = ();
+      $_gather_ref = $_aref; @{ $_aref } = ();
    }
 
    ## -------------------------------------------------------------------------
@@ -316,10 +346,10 @@ sub mce_stream (@) {
          push @_mode, exists $_[0]->{mode} ? $_[0]->{mode} : $DEFAULT_MODE;
 
          unless (ref $_code[-1] eq 'CODE') {
-            @_ = (); _croak("$_tag: 'code' is not valid");
+            @_ = (); _croak("$_tag: (code) is not valid");
          }
          if ($_mode[-1] ne 'grep' && $_mode[-1] ne 'map') {
-            @_ = (); _croak("$_tag: 'mode' is not valid");
+            @_ = (); _croak("$_tag: (mode) is not valid");
          }
       }
 
@@ -378,18 +408,20 @@ sub mce_stream (@) {
       delete $_p->{gather}      if (exists $_p->{gather});
    }
 
-   $_max_workers = int($_max_workers / @_code + 0.5) + 1
-      if (@_code > 1);
+   if (@_code > 1 && $_max_workers > 1) {
+      $_max_workers = int($_max_workers / @_code + 0.5) + 1;
+   }
 
    my $_chunk_size = MCE::Util::_parse_chunk_size(
       $CHUNK_SIZE, $_max_workers, $_params, $_input_data, scalar @_
    );
 
    if (defined $_params) {
-      $_input_data = $_params->{input_data} if (exists $_params->{input_data});
-
       if (exists $_params->{_file}) {
-         $_input_data = $_params->{_file}; delete $_params->{_file};
+         $_input_data = delete $_params->{_file};
+      }
+      else {
+         $_input_data = $_params->{input_data} if exists $_params->{input_data};
       }
    }
 
@@ -401,27 +433,30 @@ sub mce_stream (@) {
       $_MCE->shutdown() if (defined $_MCE);
 
       pop( @_queue )->DESTROY for (@_code .. @_queue);
-      push @_queue, MCE::Queue->new for (@_queue .. @_code - 2);
+
+      push @_queue, MCE::Queue->new(fast => $FAST)
+         for (@_queue .. @_code - 2);
 
       _gen_user_tasks(\@_queue, \@_code, \@_mode, \@_name, \@_wrks);
 
       my %_options = (
-         use_threads => 0, max_workers => $_max_workers, task_name => $_tag,
+         max_workers => $_max_workers, task_name => $_tag,
          user_tasks => \@_user_tasks, task_end => \&_task_end,
-         use_slurpio => 0
+         use_slurpio => 0,
       );
 
       if (defined $_params) {
-         my $_p = $_params;
+         local $_; my $_p = $_params;
 
-         foreach (keys %{ $_p }) {
+         for (keys %{ $_p }) {
+            next if ($_ eq 'sequence_run');
             next if ($_ eq 'max_workers' && ref $_p->{max_workers} eq 'ARRAY');
             next if ($_ eq 'task_name' && ref $_p->{task_name} eq 'ARRAY');
             next if ($_ eq 'input_data');
             next if ($_ eq 'chunk_size');
             next if ($_ eq 'task_end');
 
-            _croak("MCE::Stream: '$_' is not a valid constructor argument")
+            _croak("MCE::Stream: ($_) is not a valid constructor argument")
                unless (exists $MCE::_valid_fields_new{$_});
 
             $_options{$_} = $_p->{$_};
@@ -431,34 +466,51 @@ sub mce_stream (@) {
       $_MCE = MCE->new(%_options);
    }
    else {
-      ## Workers may persist after running. MCE::Stream allows options
-      ## to be passed using an anonymous hash. Therefore, lets update
-      ## the MCE instance. These options do not require respawning.
-
-      for (qw(
-         RS interval stderr_file stdout_file user_error user_output
-         job_delay submit_delay on_post_exit on_post_run user_args
-         flush_file flush_stderr flush_stdout
-      )) {
-         $_MCE->{$_} = $_params->{$_} if (exists $_params->{$_});
+      ## Workers may persist after running. Thus, updating the MCE instance.
+      ## These options do not require respawning.
+      if (defined $_params) {
+         for my $_p (qw(
+            RS interval stderr_file stdout_file user_error user_output
+            job_delay submit_delay on_post_exit on_post_run user_args
+            flush_file flush_stderr flush_stdout
+         )) {
+            $_MCE->{$_p} = $_params->{$_p} if (exists $_params->{$_p});
+         }
       }
    }
 
    ## -------------------------------------------------------------------------
 
    if (defined $_input_data) {
-      @_ = (); $_MCE->process({ chunk_size => $_chunk_size }, $_input_data);
+      @_ = ();
+      $_MCE->process({ chunk_size => $_chunk_size }, $_input_data);
+      delete $_MCE->{input_data};
    }
    elsif (scalar @_) {
       $_MCE->process({ chunk_size => $_chunk_size }, \@_);
+      delete $_MCE->{input_data};
    }
    else {
-      $_MCE->run({ chunk_size => $_chunk_size }, 0)
-         if (defined $_params && exists $_params->{sequence});
+      if (defined $_params && exists $_params->{sequence}) {
+         $_MCE->run({
+            chunk_size => $_chunk_size, sequence => $_params->{sequence}
+         }, 0);
+         if (exists $_params->{sequence_run}) {
+            delete $_params->{sequence_run};
+            delete $_params->{sequence};
+         }
+         delete $_MCE->{sequence};
+      }
    }
 
    MCE::_restore_state;
 
+   if (exists $_MCE->{_rla_return}) {
+      $MCE::MCE->{_rla_return} = delete $_MCE->{_rla_return};
+   }
+
+   finish() if ($^S);   ## shutdown if in eval state
+
    return map { @{ $_ } } delete @_tmp{ 1 .. $_order_id - 1 }
       unless (defined $_aref);
 
@@ -499,13 +551,13 @@ sub _gen_user_tasks {
 
          if (ref $_chunk_ref) {
             push @_a, ($_mode_ref->[-1] eq 'map')
-               ?  map { &$_code } @{ $_chunk_ref }
-               : grep { &$_code } @{ $_chunk_ref };
+               ?  map { &{ $_code } } @{ $_chunk_ref }
+               : grep { &{ $_code } } @{ $_chunk_ref };
          }
          else {
             push @_a, ($_mode_ref->[-1] eq 'map')
-               ?  map { &$_code } $_chunk_ref
-               : grep { &$_code } $_chunk_ref;
+               ?  map { &{ $_code } } $_chunk_ref
+               : grep { &{ $_code } } $_chunk_ref;
          }
 
          MCE->gather( (@{ $_code_ref } > 1)
@@ -536,8 +588,8 @@ sub _gen_user_tasks {
                $_chunk = MCE->thaw($_chunk);
 
                push @_a, ($_mode_ref->[$_pos] eq 'map')
-                  ?  map { &$_code } @{ $_chunk->[0] }
-                  : grep { &$_code } @{ $_chunk->[0] };
+                  ?  map { &{ $_code } } @{ $_chunk->[0] }
+                  : grep { &{ $_code } } @{ $_chunk->[0] };
 
                MCE->gather(MCE->freeze([ \@_a, $_chunk->[1] ]));
             }
@@ -567,8 +619,8 @@ sub _gen_user_tasks {
                $_chunk = MCE->thaw($_chunk);
 
                push @_a, ($_mode_ref->[0] eq 'map')
-                  ?  map { &$_code } @{ $_chunk->[0] }
-                  : grep { &$_code } @{ $_chunk->[0] };
+                  ?  map { &{ $_code } } @{ $_chunk->[0] }
+                  : grep { &{ $_code } } @{ $_chunk->[0] };
 
                MCE->gather(\@_a, $_chunk->[1]);
             }
@@ -583,12 +635,15 @@ sub _gen_user_tasks {
 
 sub _validate_number {
 
-   my $_n = $_[0]; my $_key = $_[1];
+   my ($_n, $_key) = @_;
+
+   _croak("$_tag: ($_key) is not valid") if (!defined $_n);
 
    $_n =~ s/K\z//i; $_n =~ s/M\z//i;
 
-   _croak("$_tag: '$_key' is not valid")
-      if (!looks_like_number($_n) || int($_n) != $_n || $_n < 1);
+   if (!looks_like_number($_n) || int($_n) != $_n || $_n < 1) {
+      _croak("$_tag: ($_key) is not valid");
+   }
 
    return;
 }
@@ -609,7 +664,7 @@ MCE::Stream - Parallel stream model for chaining multiple maps and greps
 
 =head1 VERSION
 
-This document describes MCE::Stream version 1.517
+This document describes MCE::Stream version 1.608
 
 =head1 SYNOPSIS
 
@@ -650,13 +705,12 @@ This document describes MCE::Stream version 1.517
 =head1 DESCRIPTION
 
 This module allows one to stream multiple map and/or grep operations in
-parallel. Code blocks run simultaneously from right-to-left. Chunk data are
-sent immediately to the next code block during processing. The results are
-appended immediately as well when passing the reference to the array.
+parallel. Code blocks run simultaneously from right-to-left. The results
+are appended immediately when providing a reference to an array.
 
    ## Appends are serialized, even out-of-order ok, but immediately.
    ## Out-of-order chunks are held temporarily until ordered chunks
-   ## arrive before appending.
+   ## arrive.
 
    mce_stream \@a, sub { $_ }, sub { $_ }, sub { $_ }, 1..10000;
 
@@ -673,64 +727,68 @@ appended immediately as well when passing the reference to the array.
    ##   ...
    ##
 
-MCE incurs a small overhead due to passing of data. Therefore, a fast code
-block will likely run faster when chaining multiple map functions natively in
-Perl. However, the overhead quickly diminishes as the complexity of the code
-increases.
+MCE incurs a small overhead due to passing of data. A fast code block will
+run faster natively when chaining multiple map functions. However, the
+overhead will likely diminish as the complexity increases for the code.
 
-   ## 0.542 secs -- baseline using the native map function
+   ## 0.334 secs -- baseline using the native map function
    my @m1 = map { $_ * 4 } map { $_ * 3 } map { $_ * 2 } 1..1000000;
 
-   ## 0.765 secs -- this is quite amazing considering data passing
+   ## 0.427 secs -- this is quite amazing considering data passing
    my @m2 = mce_stream
          sub { $_ * 4 }, sub { $_ * 3 }, sub { $_ * 2 }, 1..1000000;
 
-   ## 0.592 secs -- appends to @m3 are immediate
+   ## 0.355 secs -- appends to @m3 immediately, not after running
    my @m3; mce_stream \@m3,
          sub { $_ * 4 }, sub { $_ * 3 }, sub { $_ * 2 }, 1..1000000;
 
-The mce_stream_s function will provide better times, useful when the input data
-is simply a range of numbers. Workers generate sequences mathematically among
-themselves without any interaction from the manager process. Two arguments are
-required for mce_stream_s (begin, end). Step defaults to 1 if begin is smaller
-than end, otherwise -1.
+Even faster is mce_stream_s; useful when input data is a range of numbers.
+Workers generate sequences mathematically among themselves without any
+interaction from the manager process. Two arguments are required for
+mce_stream_s (begin, end). Step defaults to 1 if begin is smaller than end,
+otherwise -1.
 
-   ## 0.447 secs -- numbers are generated mathematically via sequence
+   ## 0.278 secs -- numbers are generated mathematically via sequence
    my @m4; mce_stream_s \@m4,
          sub { $_ * 4 }, sub { $_ * 3 }, sub { $_ * 2 }, 1, 1000000;
 
 =head1 OVERRIDING DEFAULTS
 
-The following list 6 options which may be overridden when loading the module.
+The following list 7 options which may be overridden when loading the module.
 
-   use Sereal qw(encode_sereal decode_sereal);
+   use Sereal qw( encode_sereal decode_sereal );
+   use CBOR::XS qw( encode_cbor decode_cbor );
+   use JSON::XS qw( encode_json decode_json );
 
    use MCE::Stream
-         default_mode => 'grep',               ## Default 'map'
-         max_workers  => 8,                    ## Default 'auto'
-         chunk_size   => 500,                  ## Default 'auto'
-         tmp_dir      => "/path/to/app/tmp",   ## $MCE::Signal::tmp_dir
-         freeze       => \&encode_sereal,      ## \&Storable::freeze
-         thaw         => \&decode_sereal       ## \&Storable::thaw
+         default_mode => 'grep',         ## Default 'map'
+         max_workers => 8,               ## Default 'auto'
+         chunk_size => 500,              ## Default 'auto'
+         fast => 1,                      ## Default 0 (fast queue?)
+         tmp_dir => "/path/to/app/tmp",  ## $MCE::Signal::tmp_dir
+         freeze => \&encode_sereal,      ## \&Storable::freeze
+         thaw => \&decode_sereal         ## \&Storable::thaw
    ;
 
 There is a simpler way to enable Sereal with MCE 1.5. The following will
-attempt to use Sereal if available, otherwise will default back to using
-Storable for serialization.
+attempt to use Sereal if available, otherwise defaults to Storable for
+serialization.
 
    use MCE::Stream Sereal => 1;
 
-   ## Serialization is through Sereal if available.
+   ## Serialization is by the Sereal module if available.
    my @m2 = mce_stream sub { $_ * $_ }, 1..10000;
 
 =head1 CUSTOMIZING MCE
 
 =over 3
 
-=item init
+=item MCE::Stream->init ( options )
+
+=item MCE::Stream::init { options }
 
 The init function accepts a hash of MCE options. The gather and bounds_only
-options, if specified, will be ignored due to being used internally by the
+options, if specified, are ignored due to being used internally by the
 module (not shown below).
 
    use MCE::Stream;
@@ -775,17 +833,20 @@ module (not shown below).
 
 Like with MCE::Stream::init above, MCE options may be specified using an
 anonymous hash for the first argument. Notice how both max_workers and
-task_name can take an anonymous array for setting values individually
-for each code block. Remember that MCE::Stream processes from
-right-to-left when setting the individual values.
+task_name can take an anonymous array for setting values uniquely
+for each code block.
+
+Remember that MCE::Stream processes from right-to-left when setting the
+individual values.
 
    use MCE::Stream;
 
    my @a = mce_stream {
-      max_workers => [ 2, 4, 3, ], task_name => [ 'c', 'b', 'a' ],
+      task_name   => [ 'c', 'b', 'a' ],
+      max_workers => [  2,   4,   3, ],
 
       user_end => sub {
-         my ($task_id, $task_name) = (MCE->task_id, MCE->task_name);
+         my ($mce, $task_id, $task_name) = @_;
          print "$task_id - $task_name completed\n";
       },
 
@@ -814,7 +875,7 @@ right-to-left when setting the individual values.
    2 - c ended
 
 Note that the anonymous hash, for specifying options, also comes first when
-passing the array reference.
+passing an array reference.
 
    my @a; mce_stream {
       ...
@@ -822,8 +883,9 @@ passing the array reference.
 
 =head1 API DOCUMENTATION
 
-Scripts using MCE::Stream can be written using the long or short form. The long
-form becomes relavant when mixing modes. Processing occurs from right-to-left.
+Scripts using MCE::Stream can be written using the long or short form.
+The long form becomes relevant when mixing modes. Again, processing
+occurs from right-to-left.
 
    my @m3 = mce_stream
       { mode => 'map',  code => sub { $_ * $_ } },
@@ -846,13 +908,34 @@ possibilities of passing input data into the code block.
 
 =over 3
 
+=item MCE::Stream->run ( { input_data => iterator }, sub { code } )
+
+=item mce_stream { input_data => iterator }, sub { code }
+
+An iterator reference can by specified for input_data. The only other way
+is to specify input_data via MCE::Stream::init. This prevents MCE::Stream
+from configuring the iterator reference as another user task which will
+not work.
+
+Iterators are described under "SYNTAX for INPUT_DATA" at L<MCE::Core|MCE::Core>.
+
+   MCE::Stream::init {
+      input_data => iterator
+   };
+
+   my @a = mce_stream sub { $_ * 3 }, sub { $_ * 2 };
+
+=item MCE::Stream->run ( sub { code }, list )
+
 =item mce_stream sub { code }, list
 
-Input data can be defined using a list or passing a reference to an array.
+Input data can be defined using a list.
 
    my @a = mce_stream sub { $_ * 2 }, 1..1000;
    my @b = mce_stream sub { $_ * 2 }, [ 1..1000 ];
 
+=item MCE::Stream->run_file ( sub { code }, file )
+
 =item mce_stream_f sub { code }, file
 
 The fastest of these is the /path/to/file. Workers communicate the next offset
@@ -862,7 +945,9 @@ position among themselves without any interaction from the manager process.
    my @d = mce_stream_f sub { chomp; $_ . "\r\n" }, $file_handle;
    my @e = mce_stream_f sub { chomp; $_ . "\r\n" }, \$scalar;
 
-=item mce_stream_s sub { code }, sequence
+=item MCE::Stream->run_seq ( sub { code }, $beg, $end [, $step, $fmt ] )
+
+=item mce_stream_s sub { code }, $beg, $end [, $step, $fmt ]
 
 Sequence can be defined as a list, an array reference, or a hash reference.
 The functions require both begin and end values to run. Step and format are
@@ -877,32 +962,19 @@ optional. The format is passed to sprintf (% may be omitted below).
       begin => $beg, end => $end, step => $step, format => $fmt
    };
 
-=item mce_stream { input_data => iterator }, sub { code }
-
-An iterator reference can by specified for input data. Notice the anonymous
-hash as the first argument to mce_stream. The only other way is to specify
-input_data via MCE::Stream::init. This prevents MCE::Stream from configuring
-the iterator reference as another user task which will not work.
-
-Iterators are described under "SYNTAX for INPUT_DATA" at L<MCE::Core>.
-
-   MCE::Stream::init {
-      input_data => iterator
-   };
-
-   my @a = mce_stream sub { $_ * 3 }, sub { $_ * 2 };
-
 =back
 
 =head1 MANUAL SHUTDOWN
 
 =over 3
 
-=item finish
+=item MCE::Stream->finish
 
-MCE workers remain persistent as much as possible after running. Shutdown
-occurs when the script exits. One can manually shutdown MCE by simply calling
-finish after running. This resets the MCE instance.
+=item MCE::Stream::finish
+
+Workers remain persistent as much as possible after running. Shutdown occurs
+automatically when the script terminates. Call finish when workers are no
+longer needed.
 
    use MCE::Stream;
 
@@ -918,19 +990,11 @@ finish after running. This resets the MCE instance.
 
 =head1 INDEX
 
-L<MCE>
+L<MCE|MCE>
 
 =head1 AUTHOR
 
 Mario E. Roy, S<E<lt>marioeroy AT gmail DOT comE<gt>>
 
-=head1 LICENSE
-
-This program is free software; you can redistribute it and/or modify it
-under the terms of either: the GNU General Public License as published
-by the Free Software Foundation; or the Artistic License.
-
-See L<http://dev.perl.org/licenses/> for more information.
-
 =cut
 
