@@ -9,9 +9,13 @@ package MCE::Subs;
 use strict;
 use warnings;
 
-use MCE;
+## no critic (Subroutines::ProhibitSubroutinePrototypes)
+## no critic (TestingAndDebugging::ProhibitNoStrict)
 
-our $VERSION = '1.517'; $VERSION = eval $VERSION;
+use MCE;
+use MCE::Relay;
+
+our $VERSION = '1.608';
 
 ###############################################################################
 ## ----------------------------------------------------------------------------
@@ -23,19 +27,20 @@ my $_loaded;
 
 sub import {
 
-   my $class = shift;
-   return if ($_loaded++);
+   my $_class = shift; return if ($_loaded++);
 
    my $_g_flg = 0; my $_m_flg = 0; my $_w_flg = 0;
-   my $_flag = sub { 1 }; my $_package = caller();
+   my $_flag = sub { 1 }; my $_package = caller;
 
    ## Process module arguments.
-   while (my $_arg = shift) {
-      $_g_flg = $_flag->() and next if ( $_arg =~ /^:getter$/i );
-      $_m_flg = $_flag->() and next if ( $_arg =~ /^:manager$/i );
-      $_w_flg = $_flag->() and next if ( $_arg =~ /^:worker$/i );
+   while (my $_argument = shift) {
+      my $_arg = lc $_argument;
 
-      _croak("MCE::Subs::import: '$_arg' is not a valid module argument");
+      $_g_flg = $_flag->() and next if ( $_arg eq ':getter' );
+      $_m_flg = $_flag->() and next if ( $_arg eq ':manager' );
+      $_w_flg = $_flag->() and next if ( $_arg eq ':worker' );
+
+      _croak("MCE::Subs::import: ($_argument) is not a valid module argument");
    }
 
    $_m_flg = $_w_flg = 1 if ($_m_flg + $_w_flg == 0);
@@ -58,9 +63,10 @@ sub mce_restart_worker (@) {
 }
 
 sub mce_forchunk    (@) { return $MCE::MCE->forchunk(@_); }
-sub mce_foreach     (@) { return $MCE::MCE->foreach(@_);  }
+sub mce_foreach     (@) { return $MCE::MCE->foreach(@_); }
 sub mce_forseq      (@) { return $MCE::MCE->forseq(@_); }
 sub mce_process     (@) { return $MCE::MCE->process(@_); }
+sub mce_relay_final ( ) { return $MCE::MCE->relay_final(); }
 sub mce_run         (@) { return $MCE::MCE->run(@_); }
 sub mce_send        (@) { return $MCE::MCE->send(@_); }
 sub mce_shutdown    ( ) { return $MCE::MCE->shutdown(); }
@@ -74,6 +80,8 @@ sub mce_exit        (@) { return $MCE::MCE->exit(@_); }
 sub mce_gather      (@) { return $MCE::MCE->gather(@_); }
 sub mce_last        ( ) { return $MCE::MCE->last(); }
 sub mce_next        ( ) { return $MCE::MCE->next(); }
+sub mce_relay      (;&) { return $MCE::MCE->relay(@_); }
+sub mce_relay_recv  ( ) { return $MCE::MCE->relay_recv(); }
 sub mce_sendto    (;*@) { return $MCE::MCE->sendto(@_); }
 sub mce_sync        ( ) { return $MCE::MCE->sync(); }
 sub mce_yield       ( ) { return $MCE::MCE->yield(); }
@@ -92,6 +100,7 @@ sub mce_thaw        (@) { return $MCE::MCE->{thaw}(@_); }
 sub mce_chunk_id    ( ) { return $MCE::MCE->chunk_id(); }
 sub mce_chunk_size  ( ) { return $MCE::MCE->chunk_size(); }
 sub mce_max_workers ( ) { return $MCE::MCE->max_workers(); }
+sub mce_pid         ( ) { return $MCE::MCE->pid(); }
 sub mce_sess_dir    ( ) { return $MCE::MCE->sess_dir(); }
 sub mce_task_id     ( ) { return $MCE::MCE->task_id(); }
 sub mce_task_name   ( ) { return $MCE::MCE->task_name(); }
@@ -109,8 +118,6 @@ sub mce_wid         ( ) { return $MCE::MCE->wid(); }
 sub _croak {
 
    goto &MCE::_croak;
-
-   return;
 }
 
 sub _export_subs {
@@ -128,6 +135,7 @@ sub _export_subs {
       *{ $_package . '::mce_foreach'     } = \&mce_foreach;
       *{ $_package . '::mce_forseq'      } = \&mce_forseq;
       *{ $_package . '::mce_process'     } = \&mce_process;
+      *{ $_package . '::mce_relay_final' } = \&mce_relay_final;
       *{ $_package . '::mce_run'         } = \&mce_run;
       *{ $_package . '::mce_send'        } = \&mce_send;
       *{ $_package . '::mce_shutdown'    } = \&mce_shutdown;
@@ -143,6 +151,8 @@ sub _export_subs {
       *{ $_package . '::mce_gather'      } = \&mce_gather;
       *{ $_package . '::mce_last'        } = \&mce_last;
       *{ $_package . '::mce_next'        } = \&mce_next;
+      *{ $_package . '::mce_relay'       } = \&mce_relay;
+      *{ $_package . '::mce_relay_recv'  } = \&mce_relay_recv;
       *{ $_package . '::mce_sendto'      } = \&mce_sendto;
       *{ $_package . '::mce_sync'        } = \&mce_sync;
       *{ $_package . '::mce_yield'       } = \&mce_yield;
@@ -159,12 +169,11 @@ sub _export_subs {
       *{ $_package . '::mce_thaw'        } = \&mce_thaw;
    }
 
-   ## Callable by both the manager and worker processes.
-
    if ($_g_flg) {
       *{ $_package . '::mce_chunk_id'    } = \&mce_chunk_id;
       *{ $_package . '::mce_chunk_size'  } = \&mce_chunk_size;
       *{ $_package . '::mce_max_workers' } = \&mce_max_workers;
+      *{ $_package . '::mce_pid'         } = \&mce_pid;
       *{ $_package . '::mce_sess_dir'    } = \&mce_sess_dir;
       *{ $_package . '::mce_task_id'     } = \&mce_task_id;
       *{ $_package . '::mce_task_name'   } = \&mce_task_name;
@@ -193,7 +202,7 @@ MCE::Subs - Exports functions mapped directly to MCE methods
 
 =head1 VERSION
 
-This document describes MCE::Subs version 1.517
+This document describes MCE::Subs version 1.608
 
 =head1 SYNOPSIS
 
@@ -213,8 +222,6 @@ are prototyped, therefore allowing one to call them without using parentheses.
 
    use MCE::Subs qw( :worker );
 
-   ## barrier synchronization among workers
-
    sub user_func {
       my $wid = MCE->wid;
 
@@ -242,27 +249,29 @@ to using MCE::Map, which takes care of creating a MCE instance and running.
    use MCE::Map;
    use MCE::Subs qw( :worker );
 
-   ## The following will serialize output to STDOUT as well as gather
-   ## to @a. mce_say displays $_ when called without arguments.
+   ## The following serializes output to STDOUT and gathers $_ to @a.
+   ## mce_say displays $_ when called without arguments.
 
    my @a = mce_map { mce_say; $_ } 1 .. 100;
 
    print scalar @a, "\n";
 
-Unlike the native Perl functions, print, printf, and say methods require the
-comma after the file handle.
+Unlike the native Perl functions, printf, print, and say methods require the
+comma after the glob reference or file handle.
 
-   MCE->print("STDERR", $error_msg, "\n");  ## Requires quotes around
-   MCE->say("STDERR", $error_msg);          ## the bare-word FH.
+   MCE->printf(\*STDERR, "%s\n", $error_msg);
+   MCE->print(\*STDERR, $error_msg, "\n");
+   MCE->say(\*STDERR, $error_msg);
    MCE->say($fh, $error_msg);
 
-   mce_print STDERR, $error_msg, "\n";      ## Quotes are optional
-   mce_say STDERR, $error_msg;              ## around the bare-word FH.
+   mce_printf \*STDERR, "%s\n", $error_msg;
+   mce_print \*STDERR, $error_msg, "\n";
+   mce_say \*STDERR, $error_msg;
    mce_say $fh, $error_msg;
 
 =head1 FUNCTIONS for the MANAGER PROCESS via ( :manager )
 
-MCE methods are described in L<MCE::Core>.
+MCE methods are described in L<MCE::Core|MCE::Core>.
 
 =over 3
 
@@ -278,6 +287,8 @@ MCE methods are described in L<MCE::Core>.
 
 =item mce_process
 
+=item mce_relay_final
+
 =item mce_restart_worker
 
 =item mce_run
@@ -302,7 +313,7 @@ MCE methods are described in L<MCE::Core>.
 
 =head1 FUNCTIONS for MCE WORKERS via ( :worker )
 
-MCE methods are described in L<MCE::Core>.
+MCE methods are described in L<MCE::Core|MCE::Core>.
 
 =over 3
 
@@ -324,6 +335,10 @@ MCE methods are described in L<MCE::Core>.
 
 =item mce_printf
 
+=item mce_relay
+
+=item mce_relay_recv
+
 =item mce_say
 
 =item mce_sendto
@@ -338,7 +353,7 @@ MCE methods are described in L<MCE::Core>.
 
 =head1 GETTERS for MCE ATTRIBUTES via ( :getter )
 
-MCE methods are described in L<MCE::Core>.
+MCE methods are described in L<MCE::Core|MCE::Core>.
 
 =over 3
 
@@ -348,6 +363,8 @@ MCE methods are described in L<MCE::Core>.
 
 =item mce_max_workers
 
+=item mce_pid
+
 =item mce_sess_dir
 
 =item mce_task_id
@@ -366,19 +383,11 @@ MCE methods are described in L<MCE::Core>.
 
 =head1 INDEX
 
-L<MCE>
+L<MCE|MCE>
 
 =head1 AUTHOR
 
 Mario E. Roy, S<E<lt>marioeroy AT gmail DOT comE<gt>>
 
-=head1 LICENSE
-
-This program is free software; you can redistribute it and/or modify it
-under the terms of either: the GNU General Public License as published
-by the Free Software Foundation; or the Artistic License.
-
-See L<http://dev.perl.org/licenses/> for more information.
-
 =cut
 
